# Derive predicted normal values for healthy non-smoking adults for
# pulmonary function, corrected for age, height, and gender.
# This function derives
# FEV1 predicted (Forced Expiratory Volume in 1 second) and
# FVC  predicted (Forced Vital Capacity) 
# and, if the observed FEV1 and/or FVC values are supplied, also
# FEV1PP (percent predicted) and FVCPP (percent predicted).

# Based on the paper
# Morris JF, Koski A, Johnson LC. 
# Spirometric standards for healthy nonsmoking adults. 
# Am Rev Respir Dis 1971; 103:57-67

# Central formulae:
# H: standing height (in m!)
# A: age (yr)

# Predicted normal value for FVC (liters) for healthy nonsmoking adults:
# Gender male:   FVC predicted (liters) = 4.84 + 5.83(H-1.75) - 0.025(A-45)
# Gender female: FVC predicted (liters) = 3.54 + 4.53(H-1.65) - 0.024(A-45)

# Predicted normal value for FEV1 (liters) for healthy nonsmoking adults:
# Gender male:   FEV1 predicted (liters)= 3.63 + 3.62(H-1.75) - 0.032(A-45)
# Gender female: FEV1 predicted (liters)=	2.72 + 3.50(H-1.65) - 0.025(A-45)

# Percent of the predicted normal values for FVC and FEV1 are calculated as
# 100 times the fraction of FVC and FEV1 observed divided by the 
# predicted normal value.

# Severity of COPD disease based on FEV1 percent predicted (from wikipedia)
# Mild: >= 80 percent
# Moderate: 50-79 percent
# Severe: 30-49 percent
# Very severe: < 30 or chronic respiratory failure symptoms

# Andreas Krause, Oct 2008

FEV1.predicted <- function(

	# Data frame with variables age, height, gender required
	# If the data contains FEV1, we derive FEV1 percent predicted
	# If the data contains FVC, we derive FVC percent predicted

	data=expand.grid(
		gender=c("male", "female"),
		age=seq(20, 70, by=10),
		height=seq(1.50, 1.8, by=0.1)
	),

	# specify names of variables if they differ from the default
	# Example: set gender="sex" if your variable name is sex
	age="age",
	gender="gender",
	height="height",

	FEV1="FEV1",
	FVC="FVC"
)
{
	# make gender "f"/"m" by converting to lower case and taking only the
	# first character. This way we can handle codings like male/female,
	# M/F, m/f, Male/Female, etc.
	data$gender.short <- substring(casefold(as.character(data[[gender]])), 1, 1)

	# Check anyway
	gender.ok <- data$gender.short[!is.na(data$gender.short)]
	if(any(!is.element(gender.ok, c("m", "f"))))
		stop("gender must be either \"m\" or \"f\".")

	# Check if we have all variables
	for(what in c(age, gender, height))
		if(!is.element(what, names(data)))
			stop("Ouch: No variable of name ", what, " in your data set.")

	# ---
	# Derivation of FEV1 and FVC predicted

	n <- nrow(data)
	data$FEV1.pred <- rep(NA, n)
	data$FVC.pred  <- rep(NA, n)

	m <- data$gender.short=="m"
	f <- data$gender.short=="f"

	data$FEV1.pred[m] <- 
		3.62*(data[[height]][m]-1.75) - 0.032*(data[[age]][m]-45) + 3.63
	data$FEV1.pred[f] <- 
		3.50*(data[[height]][f]-1.65) - 0.025*(data[[age]][f]-45) + 2.72

	data$FVC.pred[m]  <- 
		5.83*(data[[height]][m]-1.75) - 0.025*(data[[age]][m]-45) + 4.84
	data$FVC.pred[f]  <- 
		4.53*(data[[height]][f]-1.65) - 0.024*(data[[age]][f]-45) + 3.54

	# ---
	# Percent predicted

	if(any(names(data)==FEV1))
		data$FEV1.PP <- 100 * data[[FEV1]] / data$FEV1.pred

	if(any(names(data)==FVC))
		data$FVC <- 100 * data[[FVC]] / data$FVC.pred

	# ---
	# Clear up

	data$gender.short <- NULL	# take it out again

	return(data)

}

