# Visualize FEV1 predicted as a function of age and height.
# Call FEV1pred() without arguments to see a default graph.
# Andreas Krause, Sep 2009.

FEV1.predicted.graph <- function(
	age=seq(20, 70, by=5),
	height=seq(1.50, 2.00, by=0.05),
	gender=c("male", "female"),

	xlab=paste("age", "\n", min(age), "-", max(age), " years", sep=""),
	ylab=paste("height", "\n", format(min(height), nsmall=2), 
		"-", format(max(height), nsmall=2), " m", sep=""),
	zlab="\nFEV1\npred\n(L)",

	color=TRUE,	# color or grayscale?

	...	# other graphics arguments that will be passed unchanged
)
{
	# ---
	# Error checks

	if(any(height > 3))
		warning("Height should be given in meters. Found height > 3.")

	if(any(age < 18) | any(age > 100))
		warning("Age outside range of 18 to 100 detected.")

	if((length(age) < 2) | (length(height) < 2)) 
		stop("Covariates age and height must be of length >= 2.")

	# ---
	# Create a grid of all combinations of age and height and gender

	data <- expand.grid(
		age=age,
		height=height,
		gender=gender
	)

	# ---
	# Call the function that predicts FEV1 for all covariate combinations

	if(!exists("FEV1.predicted"))
		stop("Can not find the function FEV1.predicted!")

	FEV1.pred <- FEV1.predicted(data=data)

	# Error check

	if(any(FEV1.pred$FEV1.pred > 10))
		warning("Implausible FEV1.pred values above 10 liters")

	# Initialize output

	graphs <- list()

	# ---
	# Graph the surface(s)

	require(lattice)
	require(RColorBrewer)	# for a nice looking color palette 

	# following the Sarkar book, p. 114

	if(color)
	{
		graphs.3d.col <- colorRampPalette(brewer.pal(11, "Spectral"),
        		interpolate="spline")
		graphs.3d.col <- graphs.3d.col(200)
	} else
	{
		graphs.3d.col <- colorRampPalette(c("black", "gray", "white"),
        		interpolate="linear")
		graphs.3d.col <- graphs.3d.col(200)
	}

	graphs[["3D both genders"]] <- wireframe(
		FEV1.pred ~ age + height | gender, 
		data=FEV1.pred,
		drape=T,	# color surface
		colorkey=TRUE,	# legend

		# axis labels
		xlab=xlab,
		ylab=ylab,
		zlab=zlab,

		col.regions=graphs.3d.col,
		...
	)

	graphs[["3D male only"]] <- wireframe(
		FEV1.pred ~ age + height, 
		data=FEV1.pred,
		subset=(gender=="male"),
		drape=T,	# color surface
		colorkey=TRUE,	# legend

		# axis labels
		xlab=xlab,
		ylab=ylab,
		zlab=zlab,

		col.regions=graphs.3d.col,
		...
	)

	# ---
	# Graph the 2D profiles

	key.age <- simpleKey(
			lines=TRUE, points=FALSE,
			text=levels(factor(FEV1.pred$age)),
			side=3, columns=1,
			title="age:", cex.title=1.1,
			transparent=TRUE,
			space="right",
			between=1	# space btw columns of legend
		)

	key.height <- simpleKey(
			lines=TRUE, points=FALSE,
			text=levels(factor(format(FEV1.pred$height))),
			side=3, columns=1,
			title="body height (cm):", cex.title=1.0,
			transparent=TRUE,
			space="right",
			between=1	# space btw columns of legend
		)

	graphs[["FEV1 by age, both genders"]] <- xyplot(
		FEV1.pred ~ height | gender,
		data=FEV1.pred,
		groups=age,
		type="l",
		xlab="body height (cm)",
		ylab="FEV1 predicted (L)",
		key=key.age
	)

	graphs[["FEV1 by age, male only"]] <- xyplot(
		FEV1.pred ~ height,
		data=FEV1.pred,
		subset=(gender=="male"),
		groups=age,
		type="l",
		xlab="body height (cm)",
		ylab="FEV1 predicted (L)",
		key=key.age,
	)

	graphs[["FEV1 by height, both genders"]] <- xyplot(
		FEV1.pred ~ age | gender,
		data=FEV1.pred,
		groups=height,
		type="l",
		xlab="age (years)",
		ylab="FEV1 predicted (L)",
		key=key.height
	)

	graphs[["FEV1 by height, male only"]] <- xyplot(
		FEV1.pred ~ age,
		data=FEV1.pred,
		subset=(gender=="male"),
		groups=height,
		type="l",
		xlab="age (years)",
		ylab="FEV1 predicted (L)",
		key=key.height
	)

	# Combination of two graphs

	graphs[["combo"]] <- list(
		gr1=update(graphs[["3D male only"]],
			plot.args=list(position=c(0.0, 0, 0.48, 1.0), more=TRUE)),
		gr2=update(graphs[["FEV1 by height, male only"]],
			plot.args=list(position=c(0.5, 0.13, 1.0, 0.83), more=FALSE))
	)

	# ---
	# The End.

	return(graphs)
}

