# Graph of a biomarker over time.

biomarker.make.data <- function(
	nsubjects=12,
	dose=sample(c(20, 40, 60, 80, 100), size=nsubjects, replace=T),
	dose.time=seq(0, 170, by=1),

	# color code to map response values to colors
	# response coding: 
	# 0-stable, 1-progressive, 2-partial response, 3-full response
	colorcode=c(
		"0"="transparent", # "blue", 
		"1"="#ff3300", # red
		"2"="#FFCC00", # yellow/ orange
		"3"="#33D000"	# green
	),

	# line width used in the same way as colors above
	lwdcode=c("0"=1, "1"=2, "2"=6, "3"=10)
)
{
	set.seed(333006) # 34830)


	data <- list(
		conc= c(100, 98, 97, 99, 103, 105, 104, 106, 106, 109, 111, 113),
		time= c(  0,  1,  2,  3,   5,  7,   14,  21,  28,  35,  42,  49),
		resp= c(  0,  0,  0,  0,   2,  2,    2,   2,   2,   3,   3,   3)
	)
	
	# Second half of data (for legibility here)
	data$conc <- c(data$conc, 113, 115, 112, 106, 102, 101, 100,  96,  97,  98)
	data$time <- c(data$time,  56,  63,  70,  84,  98, 112, 126, 140, 154, 168)
	data$resp <- c(data$resp,   3,   3,   3,   3,   2,   2,   2,   1,   1,   1)

	data <- as.data.frame(data)

	# nonmem-type coding: AMT = amount (dose), MDV = missing dependent variable
	data$AMT  <- rep(0, nrow(data))
	data$MDV  <- rep(0, nrow(data))

	# dosing records
	ndoses <- length(dose.time)
	doses <- data[1:ndoses, ]
	doses$conc <- rep(0, ndoses)
	doses$time <- dose.time
	doses$resp <- rep(0, ndoses)
	doses$AMT  <- rep(dose[1], ndoses)
	doses$MDV  <- 1

	df <- rbind(data, doses)
	df <- df[order(df$time, -df$MDV), ]

	# Add the color specification to the data frame
	df$color <- colorcode[as.character(df$resp)]
	df$lwd   <-   lwdcode[as.character(df$resp)]

	df.all <- NULL

	# Create subject identifiers "001", "002", "003", etc.
	ID <- gsub(" ", "0", format(c(999, 1:nsubjects)))[-1]
	ID <- paste("101", ID, sep="-")	# study - subject

	for(i in 1:nsubjects)
		df.all <- rbind(df.all, df)

	# Add the subject ID
	df.all$ID <- rep(ID, rep(nrow(df), nsubjects))

	# doses differ for different subjects
	df.all$AMT[df.all$MDV==1] <- rep(dose, rep(ndoses, length(dose)))

	# Add a little noise
	df.all$conc <- df.all$conc * runif(nrow(df.all), 0.95, 1/0.95)

	# Add drug holidays and dose reductions
	for(who in sort(unique(df.all$ID)))
	{
		start <- sample(df.all$time[-(1:50)], size=3)
		length <- sample(1:length(dose.time)/4, size=3)

		# two periods of missed doses
		for(i in 1:2)
			df.all$AMT[df.all$ID==who & df.all$time >= start[i] & 
				df.all$time <= start[i] + length[i]] <- 0

		# one period of halved dose 
		for(i in 3)
		{
			which <- df.all$ID==who & df.all$time >= start[i] &
				df.all$time <= start[i] + length[i]
			df.all$AMT[which] <- df.all$AMT[which]/2
		}
	}

	return(df.all)
}


# ---
# Graph

biomarker.graph <- function(
	data=biomarker.make.data(nsubjects=2),
	response="conc",
	ID="ID",
	color="color",
	lwd="lwd",	# line width variable name
	time="time",

	dose="AMT",
	MDV="MDV",

	normal.range=c(100, 110),

	dose.space=0.25,	# fraction of panel allocated to dose history

	# graph arguments
	xlab="time (days)",
	ylab="concentration (ng/mL)",

	...
)
{
	require(lattice)

	# Check if all variables are present in the data
	for(what in c("ID", "response", "dose", "color", "lwd", "time"))
		if(!is.element(get(what), names(data)))
			stop(what, "=\"", get(what), "\" not in data set.")

	# ---
	# Initialize

	Y.range <- range(c(normal.range, data[[response]][!data[[MDV]]]), na.rm=TRUE)
	dose.range <- range(data[[dose]], na.rm=TRUE)

	# ---
	# Prepare the data

	measure <- data[data[[MDV]]==0, ]
	dosing  <- data[data[[MDV]]==1, ]

	Yr <- Y.range[2]-Y.range[1]

	# adjust lower limit such that the graph is expanded to fulfill
	# that the dose history covers dose.space percent of the graph.
	# Derivation: r=range of conc-time, x=dose.space, then the new
	# lower limit of the graph becomes old lower limit - r*x/(1-x)

	# The total y-axis range is expanded to create space for the dose histor

	graph.yrange <- Y.range
	graph.yrange[1] <- Y.range[1] - Yr * dose.space / (1-dose.space)

	# the range on the y-axis (scaled to the Y variable) that will
	# contain the dose history
	graph.dose.ymax <- Y.range[1]
	graph.dose.ymin <- graph.yrange[1]

	# ---
	# Function to scale dosing amounts to the y-axis of the Y variable

	scale.dose <- function(dose, ymin, ymax, dosemin, dosemax)
	{
		dscale <- ymin + (ymax-ymin)*(dose-dosemin)/(dosemax-dosemin)
		return(dscale)
	}

	# ---
	# Define the alternating axes
	# See the book by D. Sarkar, "Lattice", pp. 149/150

	axis.mine <- function(side, ...)
	{
		# cat("Axis side: ", side, "\n")
		# Init (see code for axis.default to learn some useful stuff

		# For the left and right axis, we do something customized.
		# In any other case we do the default axis.
		if(!is.element(side, c("left", "right")))
		{
			axis.default(side=side, ...)
			return()
		}

		# On the left we put the Y value axis,
		# on the right we put the dose amount axis.

		if(side=="left") what <- "Y" 
			else 
				if(side=="right") what <- "dose"
				else stop("Ouch. This should not happen.")

		# range of axis
		if(what=="Y")    Yr <- Y.range
		if(what=="dose") Yr <- dose.range

		# use 'pretty' values inside the range of data for axis ticks
		Y.pretty <- pretty(Yr, n=3)
		Y.pretty <- Y.pretty[Y.pretty >= min(Yr) & Y.pretty <= max(Yr)]

		# add one more tick at the border of conc profile and dose history
		if(what=="dose")
			Y.pretty <- unique(c(0, Y.pretty, dose.range[2]))

		# position of tickmarks on the scale of observed (conc) values

		if(what=="Y")    Y.at   <- Y.pretty
		if(what=="dose") Y.at <- scale.dose(Y.pretty, graph.dose.ymin,
			graph.dose.ymax, dose.range[1], dose.range[2])

		# For panels 1, 3, 5, etc we put axis ticks and labels,
		# for panels 2, 4, 6, etc we put only the ticks but no labels
		# since they overlap and look ugly otherwise.

		put.labels <- panel.number() %% 2 == 1

		# We now add the Y-axis or the dose axis, depending on 'what' which
		# in turn depends on the side of the axis we plot.
		panel.axis(
			side=side, 
			outside=TRUE, 
			at=Y.at,
			labels=switch(put.labels, 
				"TRUE"=Y.pretty, 
				"FALSE"=rep("", length(Y.at))
			),
			tck=0.7, 
			line.col="gray", 
			text.cex=0.75,
			check.overlap=TRUE,

			text.col=switch(what, "Y"="darkgray", "dose"="darkgray", 
				stop("Ouch"))
		)

		# horizontal lines for the dose history at Y-tickmarks
		if(what=="dose")
			panel.abline(h=Y.at, col="lightgray")

	}	# end axis function

	graph <- xyplot(
		data[[response]] ~ data[[time]] | data[[ID]],
		type="b", 
		pch="|",
		as.table=TRUE,
		xlab=xlab,
		ylab=ylab,

		..., # all other arguments passed along

		axis=axis.mine,

		scales=list(
			y=list(
				relation="free",
				limits=graph.yrange,
				alternating=2
			)
		),

		# arguments to the panel function
		fulldata=data,
		Y.line=Y.range[1],	# horizontal line at min(conc)
		normal.range=normal.range,

		panel=function(x, y, ..., 
			fulldata,
			subscripts, 
			Y.line,	# horizontal line separating profile and dose history
			normal.range
		)
		{
			# normal range as shaded area
			xnormal <- c(min(x), max(x), max(x), min(x))
			xnormal <- c(-1000, 1000, 1000, -1000)	# huge range outside the limits
				# to obtain a normal band that stretches across the entire panel
				# (and not just from min(x) to max(x))
			ynormal <- c(normal.range[1], normal.range[1], normal.range[2], 
				normal.range[2])
			panel.polygon(xnormal, ynormal, col="#ffffee")

			# show the profile
			xdata <- fulldata[subscripts, ]
			Y <- xdata[xdata[[MDV]] == 0, ]	# measurements
			D <- xdata[xdata[[MDV]] != 0, ]	# dosing records

			# the profile
			n <- nrow(Y)
			panel.segments(Y[[time]][-n], Y[[response]][-n], 
				Y[[time]][-1], Y[[response]][-1], ..., 
				lwd=Y[[lwd]], col=Y[[color]])

			# dosing history
			panel.abline(h=Y.line, col="black")	# separator profile/ dose history

			# replace each dose value by a combo of (0, dose, NA) such that we have
			# lines from 0 to the dose, then an interruption because of the NA,
			# then the next dose
			Dtime <- rep(D[[time]], rep(3, nrow(D)))
			Dvalue <- as.vector(rbind(0, D[[dose]], NA))
			Dvalue <- scale.dose(Dvalue, graph.dose.ymin,
				graph.dose.ymax, dose.range[1], dose.range[2])
			panel.lines(Dtime, Dvalue, type="l")
		}
	)

	# Color of profiles
	# Now add the dose history: see the other code.
	# Normal ranges (that vary over time!)

	return(graph)
}

