# New conc-time profile where conc can be anything (like a PD endpoint, for
# example white blood cell count), and it can be on any range like from
# 30 to 100 (as for heart rate), not necessarily starting at 0.

conc.time <- function(
	data=data.full,
	ID="ID",

	time="TIME",
	xlab=time,

	# names of variables containing the data
	Y="DV",
	PRED="PRED",
	IPRE="IPRE",

	# labels to include in the legend
	Y.label="Observed",
	PRED.label="Population Prediction",
	IPRE.label="Individual Prediction",

	# colors for data, PRED, IPRE
	Y.col="blue",
	PRED.col="red",
	IPRE.col="darkgreen",
	dosehist.col="darkred",

	# Define a column outlier and store TRUE/FALSE values in it.
	# The outliers (TRUE) are indicated by red circles for visibility.
	# Example use: data$outlier <- abs(data$WRES) > 3	
	outlier="outlier",
	outlier.pch=15,	# outliers as filled squares
	outlier.col="red",	# outliers in red

	# range of the y-axis
	Y.range=range(data[[Y]][data[[dose]]==0], na.rm=T),
	ylab=Y,

	type="p",	# points, lines, both
	y.relation="free", 	# for y-axis; "free"=individual y-scales, "fixed"

	dose="AMT",
	dose.units="mg",
	dose.range=range(data[[dose]], na.rm=T),
		# btw 0 and 1: how much of the graph is given to the dose history?
	dosehist.range=0.25,	
	...
)
{
	# ---
	# Error check

	if(dosehist.range >= 1 | dosehist.range <= 0)
		stop("dosehist.range must be between 0 and 1 and is ", dosehist.range)

	check <- c("ID", "time", "Y", "dose")

	for(i in 1:length(check))
	{
		check.par <- get(check[i])
		if(!is.element(check.par, names(data)))
			stop(check[i], "=", check.par, ": ", check.par, " not in data.")
	}

	is.measurement <- data[[dose]] == 0 | is.na(data[[dose]])
	is.dosing      <- data[[dose]] != 0 & !is.na(data[[dose]])

	dang <- !is.measurement & !is.dosing
	if(any(dang))
	{
		print(data[dang, check])
		stop("The records above seem to be neither ",
			"measurement nor dosing records.")
	}

	dang <- is.measurement & is.dosing
	if(any(dang))
	{
		print(data[dang, check])
		stop("The records above seem to be ",
			"measurement AND dosing records.")
	}

	# ---
	# Initialization

	require(lattice)

	# ---
	# Prepare the data

	measure <- data[is.measurement, ]
	dosing  <- data[is.dosing, ]

	Yr <- Y.range[2]-Y.range[1]

	# adjust lower limit such that the graph is expanded to fulfill
	# that the dose history covers dosehist.range percent of the graph.
	# Derivation: r=range of conc-time, x=dosehist.range, then the new
	# lower limit of the graph becomes old lower limit - r*x/(1-x)

	# The total y-axis range is expanded to create space for the dose history
	graph.yrange <- Y.range
	graph.yrange[1] <- Y.range[1] - Yr * dosehist.range / (1-dosehist.range)

	# the range on the y-axis (scaled to the Y variable) that will
	# contain the dose history
	graph.dose.ymax <- Y.range[1]
	graph.dose.ymin <- graph.yrange[1]

	# ---
	# Function to scale dosing amounts to the y-axis of the Y variable

	scale.dose <- function(dose, ymin, ymax, dosemin, dosemax)
	{
		dscale <- ymin + (ymax-ymin)*(dose-dosemin)/(dosemax-dosemin)
		return(dscale)
	}

	# ---
	# Define the alternating axes
	# See the book by D. Sarkar, "Lattice", pp. 149/150

	axis.mine <- function(side, ...)
	{
		# cat("Axis side: ", side, "\n")
		# Init (see code for axis.default to learn some useful stuff

		# For the left and right axis, we do something customized.
		# In any other case we do the default axis.
		if(!is.element(side, c("left", "right")))
		{
			axis.default(side=side, ...)
			return()
		}

		# On the left we put the Y value axis,
		# on the right we put the dose amount axis.

		if(side=="left") what <- "Y" 
			else 
				if(side=="right") what <- "dose"
				else stop("Ouch. This should not happen.")

		# range of axis
		if(what=="Y")    Yr <- Y.range
		if(what=="dose") Yr <- dose.range

		# use 'pretty' values inside the range of data for axis ticks
		Y.pretty <- pretty(Yr, n=3)
		Y.pretty <- Y.pretty[Y.pretty >= min(Yr) & Y.pretty <= max(Yr)]

		# add one more tick at the border of conc profile and dose history
		if(what=="dose")
			Y.pretty <- unique(c(0, Y.pretty, dose.range[2]))

		# position of tickmarks on the scale of observed (conc) values

		if(what=="Y")    Y.at   <- Y.pretty
		if(what=="dose") Y.at <- scale.dose(Y.pretty, graph.dose.ymin,
			graph.dose.ymax, dose.range[1], dose.range[2])

		# For panels 1, 3, 5, etc we put axis ticks and labels,
		# for panels 2, 4, 6, etc we put only the ticks but no labels
		# since they overlap and look ugly otherwise.

		put.labels <- panel.number() %% 2 == 1

		# We now add the Y-axis or the dose axis, depending on 'what' which
		# in turn depends on the side of the axis we plot.
		panel.axis(
			side=side, 
			outside=TRUE, 
			at=Y.at,
			labels=switch(put.labels, 
				"TRUE"=Y.pretty, 
				"FALSE"=rep("", length(Y.at))
			),
			tck=0.7, 
			line.col="gray", 
			text.cex=0.75,
			check.overlap=TRUE,

			text.col=switch(what, "Y"="darkgray", "dose"="darkgray", 
				stop("Ouch"))
		)

		# horizontal lines for the dose history at Y-tickmarks
		if(what=="dose")
			panel.abline(h=Y.at, col="lightgray")

	}	# end axis function

	# ---
	# Graph it

	graph <- xyplot(
		data[[Y]] ~ data[[time]] | format(data[[ID]]),
		alldata=data,
		Y=Y, 
		PRED=PRED,
		IPRE=IPRE,
		outlier=outlier, # TRUE/FALSE variable (!)
		time=time, ID=ID, dose=dose,

		as.table=T,
		xlab=xlab,
		ylab=ylab,
		type=type,

		outlier.pch=outlier.pch,
		outlier.col=outlier.col,

		axis=axis.mine,

		# space between panels (x-dimension)
		between=list(x=1.0),

		# distance to the right margin must be enlarged
		par.settings=list(layout.widths=list(right.padding=5)),

		...,
		scales=list(
			y=list(
				relation=y.relation,
				limits=graph.yrange,
				alternating=2,
				Yrange=Y.range,
				doserange=dose.range
			)
		),

		# panel function arguments
		Y.line=Y.range[1], 	# line at min (Y)

		# panel function
		panel=function(x, y, ..., 
			type="b",	# line type: "p"oints, "l"ines, "b"oth
			Y.line, # horizontal line at Y.line (separate values and dose hist)
			outlier.pch=17, # plot character for outliers
			outlier.col="red",	# plot color for outliers
			subscripts,
			alldata, Y, time, dose, ID
		)
		{
			D <- data[subscripts,, drop=F]
			adm  <- D[D[[dose]]!=0,, drop=F]
			meas <- D[D[[dose]]==0,, drop=F]

			# horizontal line at the boundaries of Y and dose history
			panel.abline(h=Y.line, col="black")

			# PRED and IPRE
			if(any(names(meas)==PRED))
				panel.lines(meas[, time], meas[, PRED], col=PRED.col, lwd=1)
			if(any(names(meas)==IPRE))
				panel.lines(meas[, time], meas[, IPRE], col=IPRE.col, lwd=1)

			# indicate outliers individually

			# show the data
			if(!any(names(data)==outlier))
				outlier <- rep(FALSE, nrow(meas))
			else 
				outlier <- meas[, outlier]

			D1 <- meas[!outlier, ]
			D2 <- meas[ outlier, ]

			# plot all data.
			# Note that we plot the outliers in background color (0),
			# so invisibly (they are still parts of the polgyons in contrast
			# to setting them to NA or excluding them)
			panel.points(meas[, time], meas[, Y], 
				col=ifelse(outlier, 0, Y.col), type=type)

			# mark outliers (plot over observed data)
			if(any(outlier))
				panel.points(D2[, time], D2[, Y], 
					col=outlier.col, pch=outlier.pch)

			# vertical lines for dose administrations

			dscaled <- scale.dose(adm[[dose]], 
					graph.dose.ymin, graph.dose.ymax, 
					dose.range[1], dose.range[2])

			origin <- scale.dose(dose.range[1], 
				graph.dose.ymin, graph.dose.ymax,
				dose.range[1], dose.range[2])

			if(nrow(adm) > 0)
				for(i in 1:nrow(adm))
					panel.lines(
						rep(adm [i, time], 2),
						c(origin, dscaled[i]),
						col=dosehist.col
					)
			else
				if(all(adm[, dose]==0))
					panel.text(
						mean(range(data[, time], na.rm=T)), 
						scale.dose(mean(dose.range),
							graph.dose.ymin, graph.dose.ymax,
							dose.range[1], dose.range[2]),
						"(no dose)", 
						adj=0.5, col="red"
					)
		},	# end panel function

		# add a legend (key)
		key=list(
			space="bottom",
			cex=0.7,
			between=1,	# space between columns
			columns=min(
				sum(is.element(c(Y, PRED, IPRE, outlier), names(data))), 2),
			lines=list(
				col=c(
					if(any(names(data)==Y)) Y.col,
					if(any(names(data)==outlier)) outlier.col,
					if(any(names(data)==PRED)) PRED.col,
					if(any(names(data)==IPRE)) IPRE.col
				),
				lty=c(
					if(any(names(data)==Y) &
						is.element(type, c("l", "b"))) 1 else 0, 
					if(any(names(data)==outlier)) 0,
					if(any(names(data)==PRED)) 1, 
					if(any(names(data)==IPRE)) 1
				),
				lwd=2, 
				size=2	# width of the lines
			),

			# points are used for the outliers only. 
			# Everything else becomes background color
			points=list(
				col=c(
					if(any(names(data)==Y))
						ifelse(is.element(type, c("p", "b")), Y.col, 0),
					if(any(names(data)==outlier)) outlier.col,
					if(any(names(data)==PRED)) 0,
					if(any(names(data)==IPRE)) 0
				),
				pch=c(
					if(any(names(data)==Y))
						ifelse(is.element(type, c("p", "b")), 1, 0),
					if(any(names(data)==outlier)) outlier.pch,
					if(any(names(data)==PRED)) 0,
					if(any(names(data)==IPRE)) 0
				)
			),
			text=list(
				c(
					if(any(names(data)==Y)) Y.label,
					if(any(names(data)==outlier)) "Outlier",
					if(any(names(data)==PRED)) PRED.label,
					if(any(names(data)==IPRE)) IPRE.label
				)
			)
		)
	)

	return(graph)
	
}

# testing code:

if(FALSE)
{
x <- data.full
xxx <- x[substring(x$ID, 4, 5)=="01" & x$TIHR < 15*24, ]
# xxx$PRED <- NULL
xxx$outlier <- abs(xxx$WRES) > 2
xxx <- xxx[xxx$ID > 11600, ]
graph <- ctime(Y="HR", time="TIHR", data=xxx, xlab="time (h)", 
	dosehist.range=0.20)

print(graph)
} 


