# Display of simulated exposure data (AUC) in different arrangements.

exposure <- function(
	doses=c(100, 200, 400, 800),
	groups=c("pre", "post"),
	means=(doses^0.7)/100,
	error=0.5, # multiplicative
	group2.factor=1.25,	# multiplier for AUC of group 2
	n=100, # per dose group

	seed=348390
)
{
	# ---
	# Initialization

	set.seed(seed)	# pseudo random numbers

	ndoses <- length(doses)
	ngroups <- length(groups)

	# ---
	# Simulate the data set

	x <- data.frame(
		dose=paste(
			format(rep(c(doses, doses), rep(n, ndoses*ngroups))),
			"mg"
		),
		AUC=rlnorm(
			n*ndoses*ngroups, 
			rep(c(means, means), rep(n, ndoses*ngroups)),	
			error
		),
		group=rep(groups, rep(n*ndoses, ngroups))
	)

	x$AUC <- ifelse(x$group==groups[2], x$AUC*group2.factor, x$AUC)

	# ---
	# graphics

	library(lattice)
	graphs <- list()

	graphs[["bwplot by group"]] <- bwplot(
		dose ~ AUC | group,
		lwd=2,
		as.table=TRUE,
		data=x
	)

	graphs[["bwplot by dose"]] <- bwplot(
		group ~ AUC | dose,
		lwd=2,
		layout=c(2, 2),
		as.table=TRUE,
		data=x
	)

	# ---
	# Combination of the two graphs into one object

	graphs$combo <- list(
		gr1=update(graphs[["bwplot by group"]], 
			plot.args=list(position=c(0.0, 0.0, 0.5, 1.0), more=TRUE)),
		gr2=update(graphs[["bwplot by dose"]], 
			plot.args=list(position=c(0.5, 0.0, 1.0, 1.0), more=FALSE))
	)

	# ---
	# The End.

	return(list(
		data=x,
		graphs=graphs
	))
}

