# Show for each group (e.g., dose group), means and confidence intervals
# over time.
# Call the function without arguments to see an example.
# Andreas Krause, April 2011

EDA.means.ci.groupwise <- f <- function(

	# data set (default: simulated)
	data=data.frame(
		time=rep(1:5, 60),
		BP=rnorm(300, 100, 3),
		dosegroup=rep(c(0, 50, 100), rep(100, 3)),
		sex=sample(c("male", "female"), 300, replace=TRUE) 
	),

	x="time", 
	y="BP", 
	groupvar="dosegroup", 
	z="sex", # optional conditioning variable

	interval.type="CI", 
		# "CI", "sdv" for std deviations * quantile(alpha), 
		# or "quantile" for (a, 1-a) quantiles

	# axis labels
	xlab=x,
	ylab=paste(y, " mean and ", 100*(1-2*alpha), "% ", interval.type,
		" by ", groupvar,
		sep=""),

	linetype="b", # type for means: "p"oints, "l"ines, "b"oth.

	alpha=0.05, # bounds for CI
	barwidth=0.05, # width of horizontal lines indicating the limits of a CI
	spacefactor=0.2, # space btw blocks of means/ CIs. 
		# Larger space = less dist btw successive blocks
		# Example: 0.2 corresponds to using 20% of the available space,
		# 1.0 makes blocks overlap

	colors=rep(c("red", "darkgreen", "blue", "orange", "black", "purple"), 33),

	show.data=TRUE, # show observed data?
	points.pch=1,	# point symbol for observed data

	legend.columns=5, # max no of columns for legend (per row)

	... # any other arguments
)
{
	# ---
	# Initialize

	require(lattice)

	# ---
	# Error checks

	for(what in c("x", "y", "groupvar"))
		if(!is.element(get(what), names(data)))
			stop(what, "=\"", get(what), "\" not in data set.")

	if(!missing(z))
		for(what in "z")
		if(!is.element(get(what), names(data)))
			stop(what, "=\"", get(what), "\" not in data set.")

	if(!missing(z) | missing(data))	# no z or demo mode
		use.cond <- TRUE
	else
	{
		use.cond <- FALSE
		z <- NULL
	}

	if(alpha <= 0 | alpha >= 0.5)
		stop("alpha must be between 0 and 0.5")

	# ---
	# Aggregate (means and std deviations)
	# Note that we use z (which might be NULL) and avoid an if/else clause

	data.grp <- data[, c(x, groupvar, z)] # condition by these variables

	m   <- aggregate(data[y], data.grp, mean, na.rm=TRUE)
	names(m)[names(m)==y] <- "mean"	# name is "mean" and not y

	n   <- aggregate(data[y], data.grp, function(x) sum(!is.na(x)))
	names(n)[names(n)==y] <- "n"	# name is "n" and not y

	sdv <- aggregate(data[y], data.grp, var, na.rm=TRUE)
	sdv[, y] <- sqrt(sdv[, y])
	names(sdv)[names(sdv)==y] <- "sdv"	# name is "sdv" and not y

	# combine into a single stats data frame
	stats <- merge(m, sdv, all=TRUE)
	stats <- merge(stats, n, all=TRUE)

	# derive lower and upper bounds
	ci.level <- qnorm(1-alpha)

	# Quantile derivation
	qlow  <- aggregate(data[y], data.grp, quantile,   alpha, na.rm=TRUE)
	qhigh <- aggregate(data[y], data.grp, quantile, 1-alpha, na.rm=TRUE)
	qmed  <- aggregate(data[y], data.grp, median,            na.rm=TRUE)

	names(qlow) [names(qlow) ==y] <- "ci.low"
	names(qhigh)[names(qhigh)==y] <- "ci.high"
	names(qmed) [names(qmed) ==y] <- "mean"

	if(interval.type=="CI")
	{
		stats$ci.low <- stats$m - ci.level*stats$sdv/sqrt(stats$n)
		stats$ci.high<- stats$m + ci.level*stats$sdv/sqrt(stats$n)
	} else
	if(interval.type=="sdv")
	{
		stats$ci.low <- stats$m - ci.level*stats$sdv
		stats$ci.high<- stats$m + ci.level*stats$sdv
	} else
	if(interval.type=="quantile")
	{
		stats <- merge(qmed , qlow,  all=TRUE)
		stats <- merge(stats, qhigh, all=TRUE)
	} else
	stop("Unknown interval.type=\"", interval.type, "\". Must be \"CI\" or \"sdv\".")

	# derive the x-axis coordinates for the groups by spreading them such
	# that the smallest distance of two successive x-values still does not
	# cause overlaps of vertical bars.

	x.unique <- sort(unique(stats[[x]]))
	minx <- min(x.unique[-1] - x.unique[-length(x.unique)])
	ncat <- length(unique(stats[[groupvar]]))
	stats$index <- as.numeric(factor(stats[[groupvar]])) - 1 # 0 to (n-1)

	stats$xpos <- stats[[x]] + (-max(stats$index)/2 + stats$index)/ncat

	x.unique <- unique(stats[[x]])
	diff <- outer(x.unique, x.unique, "-")
	min.diff <- min(abs(diff[row(diff) < col(diff)])) # minimum off diagonal
	# min.diff is the minimum difference between any pair of x-values

	# define the new x-axis positions:
	stats$xpos <- stats[[x]] + min.diff * spacefactor * 
		(-max(stats$index)/2 + stats$index)/ncat

	# ---
	# Graph

	require(lattice)

	# Note: we use the range of ci and the range of the x-variable to define
	# the limits of the x- and y-axis, but we actually never plot that 'data'.
	# Plotting happens on the stats data frame inside the panel function.

	form <- "stats$ci.low ~ stats$xpos"
	if(!is.null(z))
		form <- paste(form, "|", "stats[[\"", z, "\"]]", sep="")

	# derive the limits for the y-axis depending on whether or not 
	# the observed data is shown

	if(show.data)
		y.range <- range(c(stats$ci.low, stats$ci.high, data[, y]), na.rm=T)
	else
		y.range <- range(c(stats$ci.low, stats$ci.high), na.rm=T)

	y.limits <- y.range + c(-1, 1)*0.02*(max(y.range) - min(y.range))

	graph <- xyplot(as.formula(form), 
		ylim=y.limits,
		alldata=stats,	# summary statistics (mean, CI)
		data.obs=data, # observed data
		xlab=xlab,
		ylab=ylab,
		xvar=x,
		yvar=y,
		as.table=TRUE,
		linetype=linetype, 
		...,
		panel=function(x, y, ..., alldata, data.obs,
			subscripts, xvar, yvar, linetype)
		{
			D <- alldata[subscripts,, drop=FALSE]

			# grid aligned with tickmarks
			panel.grid(h=-1, v=-1, col="lightgray")

			# ---
			# observed data: extract only those records where the conditioning
			# variable, z, has the same value as the summary stats records
			# (we use the first element for comparison, they are all the same)
			# No need to subset if there is no z (single panel)

			if(!is.null(z))
			{
				which <- data.obs[, z]==D[1, z] # same value as summary stats
				data.obs <- data.obs[which, ]
			}

			# plot observed data by grouping variable

			if(show.data)
			{

				# table of grouping var, color index, x-axis position
				map <- stats[, c(groupvar, "time", "index", "xpos")]
				dupl <- duplicated(paste(map[, groupvar], map[, "time"], 
					sep="xx"))
				map <- map[!dupl, ]

				# Add color index to observed data
				obs2 <- merge(data.obs, map, all=FALSE)

				# check if we added rows to the data (indication of an error)
				if(nrow(data.obs) != nrow(obs2))
					stop("nrow(data.obs)=", nrow(data.obs), " != ",
						"nrow(obs2)=", nrow(obs2))

				# add observed data
				panel.points(obs2[, "xpos"], obs2[, yvar], pch=points.pch,
					col=colors[obs2[,"index"]+1])
			}

			# show means interconnected by grouping variable
			for(i in sort(unique(D$index)))
			{
				which <- D$index == i
				this.x <- D$xpos[which]
				this.y <- D$mean[which]
				o <- order(this.x)
				panel.points(this.x[o], this.y[o], 
					col=colors[i+1],
					type=linetype)
			}

			# plot conf intervals, line by line

			# do.bar gets 3 values of x and y (min, mean, max), 
			# derives a set of x/y-values that represents the polygon to plot,
			# and plots one vertical conf interval.
			do.bar <- function(x, y, w=barwidth, color)
			{
				if(length(x)!=3)
					stop("length(x) must be 3!")

				if(length(y)!=3)
					stop("length(x) must be 3!")

				x2 <- c(x[1]-w, x[1]+w, x[1], x[3], x[3]-w, x[3]+w)
				y2 <- c(y[1]  , y[1]  , y[1], y[3], y[3],   y[3])
				panel.lines(x2, y2, col=color)
			}
			
			for(i in sort(unique(D$index)))
			{
				D2 <- D[D$index == i, ]

				for(tm in 1:nrow(D2))
					do.bar(
						rep(D2$xpos[tm], 3), 
						c(D2$ci.low[tm], D2$mean[tm], D2$ci.high[tm]),
						color=colors[i+1]
					)
			}

		}
	)

	# Add a key (legend) to the graph

	xx <- stats[!duplicated(stats[[groupvar]]), ]
	if(missing(legend.columns))
		legend.columns <- min(nrow(xx), legend.columns)

	graph <- update(graph,
		key=list(
			text=list(levels(factor(xx[[groupvar]])), col=colors[1:nrow(xx)]),
			lines=list(col=colors[1:nrow(xx)]),
			columns=legend.columns,
			space="bottom"
		)
	)

	return(list(graph=graph, stats=stats))
}
