
# Show means/medians and associated standard errors/ confidence intervals
# to explore structure in data
# Note: data must be homogeneous. Do not pool, for example, different dose
# groups. Data can be stratified for this reason.

EDA.means.ci <- function(
	data,
	y="CONC",
	x="TIME",
	z=NULL,	# conditioning covariate, optional
	z2=NULL, # second conditioning covariate, optional

	mean.type="mean",	# "mean" or "median"
	int.type="std.err",	# "std.err" (of the mean) or "conf.int"
	int.sigma=2,	# 2 std dev/errors around the mean

	# colors
	col.sd="darkolivegreen3",	# std dev interval
	col.se="cornsilk2",	# std err interval

	# lines (optional), set lines=NULL for no lines
	hline=0,
	vline=NULL,
	line.col="lightgray",

	# Lattice graphics parameters
	type="b",
	xlab=x,
	ylab=y,
	...  # e.g., type="b", scales=list(rel="free"), etc.
)
{
	# ---
	# Initialize

	require(lattice)

	# ---
	# Check for errors

	if(missing(data))
		stop("data not specified.")

	if(!is.data.frame(data))
		data <- as.data.frame(data)

	check <- c("y", "x", if(!missing(z)) "z", if(!missing(z2)) "z2")

	for(what in check)
		if(!is.element(get(what), names(data)))
			stop(what, "=\"", get(what), "\" not found in data set.")

	# ---
	# Aggregate the data by the x-variable

	M.fun <- switch(mean.type,
		"mean"=mean,
		"median"=median,
		stop("Unknown aggregation function: mean.type=\"", mean.type, "\"")
	)

	if(missing(z))
	{
		z <- "dummy" 
		data[["dummy"]] <- rep("", nrow(data))
	}

	if(!missing(z2)) z <- c(z, z2)

	if(!missing(z))
	{
		subset <- data[, c(x, z, z2), drop=F]
		names(subset) <- c("x", "z", "z2")[1:ncol(subset)]
	}

	M <- aggregate(list(y=data[[y]]), subset, M.fun)
	M$type <- rep("M", nrow(M))

	sdv <- aggregate(list(y=data[[y]]), subset, 
		function(x) sqrt(var(x, na.rm=T)))$y

	n <- aggregate(list(y=data[[y]]), subset, function(x) sum(!is.na(x)))$y

	LOW.se  <- M
	LOW.se$y <- M$y - int.sigma * sdv / sqrt(n)
	LOW.se$type <- rep("low.se", nrow(LOW.se))

	LOW.sd  <- M
	LOW.sd$y <- M$y - int.sigma * sdv
	LOW.sd$type <- rep("low.sd", nrow(LOW.sd))

	HIGH.se <- M
	HIGH.se$y <- HIGH.se$y + int.sigma * sdv / sqrt(n)
	HIGH.se$type <- rep("high.se", nrow(HIGH.se))

	HIGH.sd <- M
	HIGH.sd$y <- HIGH.sd$y + int.sigma * sdv
	HIGH.sd$type <- rep("high.sd", nrow(HIGH.sd))

	df <- rbind(M , LOW.se)
	df <- rbind(df, LOW.sd)
	df <- rbind(df, HIGH.se)
	df <- rbind(df, HIGH.sd)

	# ---
	# put the call together

	form <- "y~x"

	if(!missing(z))
		form <- paste(form, "| format(z)", sep="")

	if(!missing(z) & !missing(z2))
		form <- paste(form, "+format(z2)", sep="")

	graph <- xyplot(as.formula(form), 
		data=df,
		D=df,
		as.table=TRUE,
		ylim=range(df$y, na.rm=TRUE),	# important to cover ci range, too
		type=type,
		xlab=xlab,
		ylab=ylab,

		...,
		panel=function(x, y, ..., subscripts, D)
		{
			D <- D[subscripts, ]
			ok <- !is.na(D$x) & !is.na(D$y)

			D2 <- D[!ok, ]
			D  <- D[ ok, ]

			# auxiliary lines first
			if(length(hline) > 0)
				panel.abline(h=hline, col=line.col)

			if(length(vline) > 0)
				panel.abline(v=vline, col=line.col)

			# widest area first (std. deviation)
			poly.x <- c(D$x[D$type=="low.sd"], rev(D$x[D$type=="high.sd"]))
			poly.y <- c(D$y[D$type=="low.sd"], rev(D$y[D$type=="high.sd"]))
			panel.polygon(poly.x, poly.y, col=col.sd, density=-1)

			# std error area
			poly.x <- c(D$x[D$type=="low.se"], rev(D$x[D$type=="high.se"]))
			poly.y <- c(D$y[D$type=="low.se"], rev(D$y[D$type=="high.se"]))
			panel.polygon(poly.x, poly.y, col=col.se, density=-1)

			panel.lines(D$x[D$type=="M"], D$y[D$type=="M"], lwd=2, col="blue", 
				...)

			# Need to think first what "n" to display.
			# No of obs, no of obs per time point, no of subjects?
			if(FALSE)
			{
				xr <- current.panel.limits()$xlim
				yr <- current.panel.limits()$ylim
				ypos <- yr[1] + 0.95*(yr[2]-yr[1])
				nobs <- sum(D$type=="M")
				panel.text(xr[2], ypos, paste("n=", nobs, " ", sep=""), adj=1)
			}
		}
	)

	return(graph)

}

