# Graphs y against x in lattice and adds optional smoothers (with legends).

EDA.xyplot.smoother <- function(
	x, y,
	z, # conditioning variable (optional)

	smoother=c("lmline", "loess"), # must refer to panel.lmline, etc.
	smoother.text=c("linear regression", "local regression (loess)"),
	smoother.col=c("darkred", "darkgreen", "orange", "darkblue", 1:99),
	smoother.lwd=3,
	smoother.lty=1:99, # line type (solid, dashed, etc.)

	points.pch=c(1, 3, 2, 4:99), # different characters for different groups
		# (only applicable if groups=... is specified)
	points.col=c("red", "blue", 1:99), # color for points
	points.cex=rep(1, 99), # character size for points/symbols

	key.columns=length(smoother), # no of columns for key/legend

	show.grid=TRUE, # gray grid in background?
	...	
		# graphics arguments to xyplot such as main, xlab, ylab, etc.
		# Note: groups=xx is another possibility to use different 
		# colors/symbols for different treatment groups
)
{
	# ---
	# Initialization

	require(lattice)

	nsmoothers <- length(smoother)

	# Extract only the colors and line types used (for the legend/key)
	smoother.col <- smoother.col[1:nsmoothers]
	smoother.lty <- smoother.lty[1:nsmoothers]

	grp <- list(...)$groups
	grp.def <- length(grp) > 0

	if(!is.factor(grp)) grp <- factor(grp) # factor for group-wise plotting

	if(grp.def)
		ngroups <- length(unique(grp))
	else 
		ngroups <- 1 # no groups (equal to one group)

	# keep only the symbols, colors, etc. actually used (for legend/key)
	points.col <- points.col[1:ngroups]
	points.cex <- points.cex[1:ngroups]
	points.pch <- points.pch[1:ngroups]

	# ---
	# Error check

	for(what.smooth in smoother)
		if(!exists(paste("panel.", what.smooth, sep="")))
			stop("No panel function found for smoother ", what.smooth)

	# retrieve panel functions (for each smoother): must exist
	panels <- lapply(as.list(paste("panel.", smoother, sep="")), get)

	# ---
	# Graphics

	form <- "y ~ x"
	if(!missing(z)) form <- paste(form, "| z")

	x <- x
	y <- y
	if(!missing(z)) z <- z

	# put the key (legend) together

	key <- list()
	key$space <- "bottom"	# position of key in graph
	key$columns <- key.columns

	col.invisible <- 0 # background color

	# key for data groups (if any)

	if(grp.def)
	{
		key$points <- list(col=points.col, pch=points.pch)
		key$text   <- list(levels(grp), col=points.col)
		key$lines  <- list(
			col=rep(col.invisible, ngroups), # (bg color)
			lty=rep(1, ngroups),
			lwd=rep(1, ngroups)
		)
	}

	# smoother legend
	key$points$col <- c(key$points$col, rep(col.invisible, nsmoothers))
	key$points$pch <- c(key$points$pch, rep(NA, nsmoothers))

	key$lines$col  <- c(key$lines$col, smoother.col)
	key$lines$lty  <- c(key$lines$lty, smoother.lty)
	key$lines$lwd  <- c(key$lines$lwd, smoother.lwd)
	
	key$text[[1]]  <- c(key$text[[1]], smoother.text)
	key$text$col   <- c(key$text$col,  smoother.col)

	graph <- xyplot(
		as.formula(form),

		..., # any other arguments passed

		panel=function(x, y, ...)
		{
			panel.grid(h=-1, v=-1, col="lightgray")
			panel.xyplot(x, y, ..., pch=points.pch, col=points.col, 
				cex=points.cex)

			# add fit for each smoother (use the panel functions)
			for(i in 1:length(panels))
				suppressWarnings(
					panels[[i]](x, y, ..., 
					lwd=smoother.lwd, 
					col=smoother.col[i],
					lty=smoother.lty[i]
				))
			# warnings are suppressed in particular for loess fits with
			# tons of warnings on sparse data
		},

		key=key
	)

	return(graph)

}


