
# ---
# Confidence intervals from bootstrap samples and fits

Emax.bootstrap.ci <- function(
	data, 
	xvar, yvar,
	bs.n, # number of bootstrap samples
	x.pred, # x-values to predict y for

	model,
	include.E0, 
	include.gamma,
	Emax.fun, 

	alpha, # CI quantile

	nls.call, # formula passed to nls function (as characters, e.g. "y=Emax(...)")
	nls.algo="port",
	start,	# starting values (for non-Emax models only)
	linear.fun, loglinear.fun,


	debug.bs=FALSE	# debug starting values routine (graphs and browser)?
)
{
	df <- data.frame(x.fit=x.pred)

	bs.pred <- matrix(-999, length(x.pred), bs.n) # init bootstrap pred values
	fit.bs <- list()

	for(i in 1:bs.n)
	{
		# sample indices and subset data matrix
		bs.indices <- sample(1:nrow(data), size=nrow(data), replace=TRUE)
		x.bs <- data[bs.indices, ]

		# derive starting values based on this current bootstrap sample
		# or use overall starting values (see arguments to this function)

		if(model=="Emax")
			start <- Emax.get.starting.values(x=x.bs[, "x.fit"], y=x.bs[, yvar], 
				include.E0=include.E0, include.gamma=include.gamma,
				debug=debug.bs)
		else
			start=start	# for other types of models, starting values are supplied

		# fit and predict at x.pred
		# Each non-converged fit produces a warning, and there can be many.
		# We issue a single warning later, stating that x/y runs 
		# did not converge, and suppress the possibly numerous warnings here.

		# Show data and initial values for fitting
		# plot(y ~ x, data=x.bs)
		# points(x.bs[, x], 
		#   Emax.fun(x.bs[, x], E0=0.9631, EC50=29.19, Emax=0.778), col="red")
		# xl <- loess(y~x, data=x.bs)
		# points(xl$x, fitted(xl), col="green")

		fit.bs[[i]] <- suppressWarnings(
			nls(as.formula(nls.call), data=x.bs, trace=FALSE, 
				start=start,
				algorithm=nls.algo, control=list(warnOnly=TRUE))
		)

		bs.pred[, i] <- predict(fit.bs[[i]], newdata=df, 
			type="response", se.fit=TRUE)
	}

	# ---
	# check if all fits converged.
	# NOTE: This works with algorithm "port" only. See help page for nls (!).

	# Note: in some cases nls reports to not have converged but all 
	# non-converged fits that I have seen so far look reasonable. 
	# Thus, we leave them in but issue a warning on the command line.

	ouch <- sapply(fit.bs, function(x) x$convergence != 0)	
		# flag non-convergence

	if(any(ouch))
	{
		str <- paste("Emax (", yvar, "~", xvar, "): ", 
			sum(ouch), "/", length(ouch), 
			# " bootstrap fits taken out for non-convergence. Potential bias.",
			" bootstrap fits report non-convergence (left in though).",
			sep="")

		warning(str)

		### Leave the non-converged fits in:
		bs.pred <- bs.pred[, !ouch, drop=FALSE]
		bs.n <- sum(!ouch)	# adjust bootstrap number of samples
	}

	# Add the information to the subtitle of the graph
	info <- paste(
		sum(!ouch), " bootstrap fits, ", 
		sum(!ouch), "/", length(ouch), " converged",
		sep="")

	# ---
	# Derive confidence boundaries
	# alpha MUST be in [0, 0.5].
	# We employ na.rm=T for cases where x.pred is negative and 
	# thus the predicted y NA.

	if(ncol(bs.pred)==0)
	{
		# none of the fits converged (!)
		df$ci.lo <- rep(NA, nrow(df))
		df$ci.hi <- rep(NA, nrow(df))

		warning("None of the bootstrap sample fits converged!")
	} else
	{
		df$ci.lo <- apply(bs.pred, 1, quantile,   alpha/2, na.rm=TRUE)
		df$ci.hi <- apply(bs.pred, 1, quantile, 1-alpha/2, na.rm=TRUE)
	}

	# ---
	# Parametric std errors
	# Calculate standard deviations and derive CI as 
	# qt(c(alpha/2, 1-alpha/2), df=...)
	# Generally yields a smoother curve for the CI but assumes that std 
	# errors are symmetric. This approach can produce illegal values such 
	# as negative values for concentrations whereas the quantile based 
	# approach cannot.
	# For df, see the bootstrap code in the S1P B201 project.
	# ---
	# NOTE: std errors cannot be obtained from predict.nls.
	# See the documentation and compare to predict.glm with se.fit=TRUE

	# code to be written if desired

	return(list(df=df, info=info))

}

