
# ---
# Routine to fit an Emax model and graph it.
# Show concentration-response on top, dose-concentration as bwplots below.
# AK, Sep/Oct 2011.

# Notes: 
# Call the function without any arguments to get to see an example.
# The function returns several graphs. Save those to file or 
#   use windows(record=TRUE) to be able to scroll back in graphics window.

# Change history:

# Extension to indicate particular fractions of Emax (EC20, EC50, EC80) by 
# horizontal/vertical lines.
# AK, 3, Feb 2012. 

# Extension to align both graphs at the x-axis tickmarks.
# AK, Feb 8, 2012.

# Added fitting of linear and loglinear model (CIs from bootstrapping).
# AK, Mar 7, 2012.

# Added linear and loglinear models without intercept.
# AK, Mar 21, 2012

# Added handling of missing values by introduction of new argument, na.action.
# Effects (EC20, EC50, EC80) moved to the right-hand side axis (from the left).
# Color defaults changed to use colorpalette, ignoring lattice global settings.
# Fixed a small bug in the Emax inverse function that showed if gamma 
# does not equal 1.
# AK, May 14, 2012

# Log concentration added as option x.logscale=FALSE (default).
# Routine for starting values improved to estimate gamma instead of 
# setting it to 1.
# Bug in bootstrap fixed that starting values were based on obs. data 
# instead of bootstrap sample.
# AK, May 16, 2012

# Starting values heuristics changed to be based on loess estimates now.
# AK, May 17, 2012

# Log scale option split into two options: logscale.fit and logscale.plot.
# Discussion with Dan Weiner, Pharsight, showed that Winnonlin/Phoenix 
# always fits on the linear scale, even if plots are shown semi-logarithmically.
# This is not consistent, the semi-log scale distorts the leverage of a few
# extremely large concentrations. Thus, we offer the option of also fitting on
# the log scale. Note however that in particular the Hill coefficient, gamma,
# can be very different from 1 even if the curve looks smooth on a 
# semi-log graph!
# Interpretation/intuition for gamma coefficients is thus slightly difficult.

Emax.fit.and.plot <- function(

	# observed data: default simulated data set
	data=Emax.simdata(n=sim.n),
	sim.n=150,

	# variable names within the data set
	y="response",
	x="concentration",
	dosegroup="dose",

	# what model to use?
	model="Emax", 
		# options: 
		# "Emax", 
		# "linear1", "linear2", # without and with intercept
		# "loglinear1", "loglinear2" # without and with intercept

	# options for the Emax model:
	include.E0=TRUE,
	include.gamma=FALSE,

	# nls options
	algorithm="port", 
		# Port seems more robust than others. See the nls documentation.
		# Check carefully if you use another algorithm.

	# Starting values for fitting the model to the data
	# if none are supplied here, there will be simple heuristics used 
	# to obtain starting values in the body of the function.
	# the port algorithm seems to require user-supplied starting values.
	# Thus, they do not need to be specified (unless the fits do not converge).

	start,

	# show iteration steps for the model?
	quiet.nls=TRUE,

	# show our own messages?
	quiet=FALSE,

	# NA handling
	na.action=na.exclude,
	na.warn=TRUE,	# warn if NA values are detected?

	# confidence interval options
	show.CI=TRUE,
	bs.n=100,
	alpha=0.9,
	lines.col="darkgreen", 
	lines.lwd=2, # line width

	# Log scale for fitting, i.e., y=f(log(conc))?
	# Log scale for plotting is just for plotting, fitting can be executed
	# on linear and on log scale data.
	# We use log10 throughout for legibility of values.

	logscale.fit=FALSE,
	logscale.plot=logscale.fit,

	# x-values to predict for
	x.pred = seq(min(data$x.fit, na.rm=T), max(data$x.fit, na.rm=T), length=100), 

	# axis tickmark positions and labels
	xaxis.at=Emax.xaxis.at(data[, "x.plot"], logscale=logscale.plot, xlim=xlim),
	xaxis.lab=if(logscale.plot) 10^xaxis.at else xaxis.at,
	
	# Graph limits
	# orig: xlim=Emax.xlim(data[[x]], logscale=(logscale.plot & !logscale.fit), npoints=12),
	xlim=Emax.xlim(data[, "x.plot"], logscale=F, npoints=12),
	ylim=Emax.xlim(data[[y]], logscale=FALSE, n=12), # range(pretty(data[[y]], n=12)),

	# The following strings are used to align the x-axes of both graphs.
	# These defaults work well for the default data set but might not for your
	# data set. Specify a longer string to move a graph (and the x-axis) to 
	# the right.
	align.str.xy="123456",
	align.str.dose="23456",

	# indicate the following individual y-values with corresponding x-values
	y.indicate=NULL, # example: c(1.2, 1.5),

	y.effect.indicate=c(0.0, 0.2, 0.5, 0.8, 1.0),
		# same as above but now you cannot indicate particular 
		# y-values but derived effects as percentages of Emax, 
		# e.g., 0.8 corresponds to 80% of the maximum effect.
		# Set to NULL if you do not want anything shown.
		# Example: y.effect.indicate=c(0.5, 0.8) 
		# shows 50% and 80% of max effect

	# colors for indicator lines and text
	col.indicator="orange",	# faint to not dominate the picture
	col.indicator.text=col.indicator,

	trunc.yaxis=FALSE, 
		# if trunc.yaxis=TRUE, the y-axis range is defined by the range of the 
		# fit and confidence interval.
		# if trunc.yaxis=FALSE, it is defined by the data range 
		#   (see definition of yaxis.at)
		# This option is useful if the data contains extreme values (outliers)
	yaxis.at=if(trunc.yaxis) pretty(c(df$ci.lo, df$ci.hi), n=10) else 
		pretty(ylim, n=12),
		# yaxis.at defines the y-axis tickmarks. Note that this default 
		# setting makes use of R's lazy evaluation: df is defined in the body 
		# of the function and yaxis.at is evaluated at its first use - 
		# after df is defined (and not right here and instantaneously).

	# plot options for data
	points.pch=1:25, # plotting character by dose group
		# Can be a character, e.g., "x" or "o"
		# for the same symbol for all dose groups use rep(1, 99)
	# points.col=c(trellis.par.get("superpose.symbol")$col, 1:999),
	points.col=colorRampPalette(c("blue", "red", "darkgreen"))(
		length(unique(data[, dosegroup]))),

	# Labels for x-axis, y-axis (two of them), main title and subtitle
	xlab=x,
	ylab=y,
	ylab.dosegroup="dose (mg)", 
	main=Emax.main(model=model, xvar=x, yvar=y, E0=E0, EC50=EC50, Emax=Emax, gamma=gamma, 
		alpha=alpha, bs.n=bs.n, include.E0=include.E0, include.gamma=include.gamma),

	sub="",	# subtitle (underneath dose-conc boxplots)

	# random number seed
	random.seed=4245,

	# debugging mode (plotting and browsing)
	debug=FALSE,
	debug.bs=debug,	# debug starting values for each (!) bs sample?

	...
)
{
	# ---
	# Initialization

	if(quiet)
		ccat <- function(...) {}
	else
		ccat <- function(...) cat(..., "\n", sep="")

	# ---
	# Startup message

	ccat("\n----------------------------------------------------------")
	ccat("Model fitting  : ", y, "~", x)
	ccat("Model          : ", model)

	if(model=="Emax")
	{
	ccat("E0 included    : ", include.E0)
	ccat("Gamma included : ", include.gamma)
	ccat("CI level       : ", alpha)
	}
	ccat("----------------------------------------------------------\n")

	# ---
	# Error checking

	for(what in c("x", "y", "dosegroup"))
		if(!is.element(get(what), names(data)))
			stop(what, "=\"", get(what), "\" not in data set.")

	for(fun in c("modeling.compare.xy", "EDA.xyplot.smoother"))
	if(!exists(fun))
		warning("Function ", fun, " not found (see the libraries). ",
			" Omitting this graph.")

	ouch <- y.effect.indicate < 0 | y.effect.indicate > 1 
	if(any(ouch))
		stop("y.effect.indicate outside [0, 1].")

	# ---
	# Initialization

	library(MASS)
	library(lattice)

	if(abs(alpha) > 1)
		stop("alpha must be in [0, 1]")
	alpha <- min(alpha, 1-alpha)	# vital for CI

	set.seed(random.seed)

	# Take care of missing values (NA)
	
	miss <- is.na(data[, x]) | is.na(data[, y]) | is.na(data[, dosegroup])
	if(any(miss))
	{
		if(na.warn)
			warning(sum(miss), "/", length(miss), 
				" records with missing values dropped.")
		data <- na.action(data)
	}

	# if logscale is used, check that all values are positive.
	if(logscale.fit)
	{
		OK <- data[, x] > 0
		if(any(!OK))
		{
			warning(sum(!OK), "/", length(OK), 
				" x-values <= 0 removed since logscale.fit=T")
			data <- data[OK, ]
		}

		data$x.fit <- log10(data[, x])
	}
	else
		data$x.fit <- data[, x]

	if(logscale.plot)
		data$x.plot <- log10(data[, x])
	else
		data$x.plot <- data[, x]

	if(logscale.plot & any(data[, x] <= 0))
		warning("log scale requested for plotting but data contain ",
			"negative values.")

	if((logscale.plot | logscale.fit) & any(x.pred <= 0))
	{
		ouch <- x.pred <= 0
		x.pred <- x.pred[!ouch]

		warning("log scale requested for plotting but x.pred contains ",
			sum(ouch), "/", length(ouch), " negative values (removed).")
	}

	# ---
	# Emax functions: define which function (parameters) to use

	Emax.E0.gamma <- function(x, E0, Emax, EC50, gamma=1)
		E0 + Emax * (x^gamma) / ( (EC50^gamma) + (x^gamma))

	Emax.E0 <- function(x, E0, Emax, EC50)
		E0 + Emax * x / ( EC50 + x)

	Emax.gamma <- function(x, Emax, EC50, gamma)
		Emax * (x^gamma) / ( (EC50^gamma) + (x^gamma))

	Emax.basic <- function(x, Emax, EC50)
		Emax * x / (EC50 + x)

	Emax.fun <- NULL
	if( include.E0 &  include.gamma) Emax.fun <- Emax.E0.gamma
	if( include.E0 & !include.gamma) Emax.fun <- Emax.E0
	if(!include.E0 &  include.gamma) Emax.fun <- Emax.gamma
	if(!include.E0 & !include.gamma) Emax.fun <- Emax.basic
	if(is.null(Emax.fun)) stop("Emax.fun not defined.")

	# ---
	# Emax functions: define the call to the Emax function (for optimization)

	call <- NULL

	if(model=="Emax")
	{
		if( include.E0 &  include.gamma) 
			call <- "y ~ Emax.fun(x.fit, E0, Emax, EC50, gamma)"
		if( include.E0 & !include.gamma) 
			call <- "y ~ Emax.fun(x.fit, E0, Emax, EC50)"
		if(!include.E0 &  include.gamma) 
			call <- "y ~ Emax.fun(x.fit, Emax, EC50, gamma)"
		if(!include.E0 & !include.gamma) 
			call <- "y ~ Emax.fun(x.fit, Emax, EC50)"
		if(is.null(call)) stop("call not defined.")
	}
	
	if(model=="linear1")
	{
		linear.fun <- function(x, b) b*x
		call <- "y ~ linear.fun(x.fit, b)"
	}

	if(model=="linear2")
	{
		linear.fun <- function(x, a, b) a+b*x
		call <- "y ~ linear.fun(x.fit, a, b)"
	}

	if(model=="loglinear1")
	{
		loglinear.fun <- function(x, b) b*log(x)
		call <- "y ~ loglinear.fun(x.fit, b)"
	}

	if(model=="loglinear2")
	{
		loglinear.fun <- function(x, a, b) a+b*log(x)
		call <- "y ~ loglinear.fun(x.fit, a, b)"
	}

	if(is.null(call))
		stop("No idea what to do with model ", model, "!")

	# ---
	# Extract the data to use

	data$x <- data[, x]
	data$y <- data[, y]

	# ---
	# Fitting the model
	# Note that the starting values are passed to this function as arguments
	# Note: not supplying starting values results in 
	# "Error: Emax.fun not found!"
	# Example (that stops): fit <- nls(as.formula(call), data=data)

	if(missing(start))
		if(model=="Emax")
		start <- Emax.get.starting.values(x=data$x.fit, y=data[, y], 
			include.E0=include.E0, include.gamma=include.gamma,
			Emax.fun=Emax.fun,
			debug=debug)
		else
		if(model=="linear1") start <- list(b=1)
		else
		if(model=="linear2") start <- list(a=0, b=1)
		else
		if(model=="loglinear1") start <- list(b=1)
		else
		if(model=="loglinear2") start <- list(a=0, b=1)

	# ---
	# Model fit (!)

	fit <- suppressWarnings(
		nls(as.formula(call), data=data, trace=!quiet.nls, start=start,
		algorithm=algorithm, control=list(warnOnly=TRUE))
	)

	# ---
	# Predict y for given x-values

	df <- data.frame(x.fit=x.pred)

	pred <- predict(fit, newdata=df, type="response", se.fit=TRUE)

	df$pred  <- pred
	df$ci.lo <- pred # initialize. Will be overwritten when CIs are calculated.
	df$ci.hi <- pred

	if(logscale.plot) df$x.plot <- log10(df$x.fit)
	else df$x.plot <- df$x.fit

	df$x.plot <- x.pred
	if(logscale.fit & !logscale.plot)
		df$x.plot <- 10^df$x.plot
	if(!logscale.fit & logscale.plot)
		df$x.plot <- log10(df$x.plot)

	# Calculate the y-values that correspond to the desired fraction of Emax
	coeffs <- coefficients(fit)

	# Derive particular x-values of interest, y where y=0.8*Emax, for example
	if(model=="Emax" & length(y.effect.indicate) > 0)
	{
		y.ind2 <- coeffs["E0"] + y.effect.indicate*coeffs["Emax"]
		x.ind2 <- Emax.inv(y=y.ind2, E0=coeffs["E0"], EC50=coeffs["EC50"], 
			Emax=coeffs["Emax"], gamma=coeffs["gamma"])
		str.ind2 <- paste(y.effect.indicate*100, "%", sep="")
	} else
	{
		x.ind2 <- NULL
		y.ind2 <- NULL
		str.ind2 <- NULL
	}

	if(logscale.plot & model=="Emax")
		x.ind2 <- log10(x.ind2)	# for plotting purposes

	# y-values supplied directly to this function
	if(model=="Emax" & length(y.indicate) > 0)
	{
		x.indicate <- Emax.inv(y=y.indicate, E0=coeffs["E0"], 
			EC50=coeffs["EC50"], Emax=coeffs["Emax"], gamma=coeffs["gamma"])
	} else x.indicate <- NULL

	# ---
	# Confidence intervals from bootstrap samples and fits

	ccat("Bootstrapping (", bs.n, " samples).")

	bs <- Emax.bootstrap.ci(data=data, bs.n=bs.n, xvar="x.fit", yvar=y, x.pred=x.pred, 
		model=model, include.E0=include.E0, include.gamma=include.gamma,
		nls.call=call, nls.algo=algorithm,
		alpha=alpha,
		Emax.fun=Emax.fun, linear.fun=linear.fun, loglinear.fun=loglinear.fun,
		start=start, # starting values for non-Emax functions
		debug.bs=debug.bs)

	# Store bs results away:
	df$ci.lo <- bs$df$ci.lo
	df$ci.hi <- bs$df$ci.hi
	sub <- paste(sub, bs$info, sep="\n") # subtitle for graph, information of how many bs samples converged

	# ---
	# Create a data frame for plotting
	# Note: we plot the model fit and supply the data as additional
	# argument to xyplot.

	df.plot <- data.frame(
		y=c(df$pred, df$ci.lo, df$ci.hi),
		x.plot=rep(df$x.plot, 3),
		what=rep(c("m", "l", "h"), rep(nrow(df), 3))
	)

	# ---
	# Graph of model fit, CI and data

	graph.model <- Emax.xyplot(yvar=y, xvar="x.fit", x=data[, "x.plot"], y=data[, y], df.plot=df.plot,
		grps=data[, dosegroup],
		xaxis.at=xaxis.at, xaxis.lab=xaxis.lab, yaxis.at=yaxis.at, 
		col.indicator=col.indicator, col.indicator.text=col.indicator.text, 
		lines.col=lines.col, lines.lwd=lines.lwd, points.col=points.col, points.pch=points.pch,   
		main=main, 
		x.indicate=x.indicate, y.indicate=y.indicate, x.ind2=x.ind2, y.ind2=y.ind2,    
		str.ind2=str.ind2, 
		logscale.plot=logscale.plot, logscale.fit=logscale.fit,  
		xlim=xlim, ylim=ylim, 
		align.str.xy=align.str.xy, 
		ylab=ylab
	)

	# ---
	# Graph of concentration by dose group

	graph.dose <- xyplot(
		factor(format(data[[dosegroup]])) ~ data[, "x.plot"],
		xlab=xlab, 
		ylab=list(ylab.dosegroup), 
		sub=sub,	# subtitle

		# sync settings for this x-axis with settings for xyplot x-axis
		scales=list(
			# sync this x-axis with the xyplot x-axis (!)
			x=list(at=xaxis.at, lab=xaxis.lab, limits=xlim),
			# include a long dummy label to align the y-axes of both graphs(!)
			y=list(
				at=1:length(levels(factor(format(data[[dosegroup]])))), 
				lab=format(c(align.str.dose,
						levels(factor(format(data[[dosegroup]]))) 
					),
					justify="right")[-1]
			)
		),

		plot.args=list(position=c(0, 0.0, 1, 0.33), more=FALSE),
		box.ratio=3,	 # higher boxes
		col.points=points.col, # col of middle points
		pch.points=points.pch, # plot character for dots
		panel=function(x, y, ..., col.points, pch.points)
		{
			# horizontal lines at doses
			panel.abline(h=unique(y), col="lightgray")

			# vertical lines at x-axis tickmarks
			panel.abline(v=xaxis.at, col="lightgray")

			# draw the bwplot on top of everything else
			panel.bwplot(x, y, ..., col=col.points, pch=pch.points)
		}
	)

	# ---
	# Graph of raw data and smoothers/fits

	if(!exists("EDA.xyplot.smoother"))
		graph.data <- NULL

	else
		graph.data <- EDA.xyplot.smoother(data[[x]], data[[y]], 
			xlab=xlab, ylab=ylab)

	# ---
	# Graph of observed vs predicted

	y.obs <- data[[y]][!is.na(data[[y]]) & !is.na(data[[x]])]
	y.pred <- fitted(fit)

	y.obs [y.obs  > 10] <- 11
	y.pred[y.pred > 10] <- 11

	# Add a graph of observed vs predicted 
	# (if the function exists in the search path)

	if(!exists("modeling.compare.xy"))
		graph.obs.pred <- NULL
	else
	graph.obs.pred <- modeling.compare.xy(
		data=data.frame(y.obs =y.obs, y.pred=y.pred, dgrp=data[, dosegroup]),
		x="y.pred", y="y.obs", 
		pch=points.pch[factor(data[, dosegroup])],
		col=points.col[factor(data[, dosegroup])],
		xlab=paste(ylab, "predicted"), # ylab since we show y obs vs pred.
		ylab=paste(ylab, "observed"),
		key=list(
			points=list(
				pch=points.pch[factor(unique(data[, dosegroup]))],
				col=sort(unique(points.col[factor(data[, dosegroup])]))
			),
			text=list(
				lab=format(unique(data[, dosegroup])), 
				col=sort(unique(points.col[factor(data[, dosegroup])]))
			),
			space="bottom",
			columns=length(unique(data[, dosegroup])) # single row only
		)
	)

	# ---
	# The End.

	return(list(

		# the data that is used for plotting (kind of lengthy)
		# pred=df,	# matrix of values
		# plot.df=df.plot, # lattice df arrangement of values

		fit=fit,
		fit.summary=summary(fit),

		graphs=list(
			data=graph.data, 
			obs.pred=graph.obs.pred,
			model.fit=list(model=graph.model, dose.conc=graph.dose)
		)
	))
}


