
# A good guess of starting values is required to get nls to converge properly for Emax modeling.
# This function guesses E0, EC50, Emax, and gamma based on a loess fit to the data.


Emax.get.starting.values <- function(x, y, include.E0, include.gamma,
	Emax.fun,
	debug=FALSE	# show graphics and start debugger?
)
{
	start <- list()

	# Use loess fit as starting point. 
	# Do a nonparametric fit and evaluate at grid points.

	Lfit <- loess(y~x)
	xcand <- seq(min(x), max(x), length=50)
	Lpred <- predict(Lfit, newdata=data.frame(x=xcand))

	# plot all candidate profiles for checking
	if(debug)
	{
		plot(x, y, sub="green: loess, red: emax w/starting values",
			xlab=deparse(substitute(x)),
			ylab=deparse(substitute(y))
		)
		lines(xcand, Lpred, col="darkgreen", lwd=2)
	}

	# Guess if Emax is positive or negative
	xsmall <- rank(x) < length(x)/2
	xlarge <- rank(x) > length(x)/2
	Emax.pos <- (mean(y[xsmall]) < mean(y[xlarge]))

	if(Emax.pos)
	{
		start$E0   <- min(Lpred)
		start$Emax <- max(Lpred) - start$E0
	}
	else
	{
		start$E0   <- max(Lpred)
		start$Emax <- min(Lpred) - start$E0
	}

	# EC50 estimate is the x-value where y is closest to 
	# the middle btw E0 and Emax
	diff <- abs(Lpred-(start$Emax+2*start$E0)/2)
	is.min <- diff==min(diff)
	start$EC50 <- xcand[is.min][1] 

	# gamma starting values off the mark can easily cause the nls to
	# abort (crash). We spend some more resources on finding a good
	# starting value therefore.
	# Algorithm: test a set of gamma values and take the one 
	# with the smallest sum of squared residuals.

	if(include.gamma)
	{
		gammas <- c(1/(10:1), 2:10)
		SS <- rep(NA, length(gammas))

		for(i in 1:length(gammas))
		{
			if(include.E0)
				yest <- Emax.fun(x, E0=start$E0, Emax=start$Emax, 
					EC50=start$EC50, gamma=gammas[i])
			else
				yest <- Emax.fun(x, Emax=start$Emax, 
					EC50=start$EC50, gamma=gammas[i])

			SS[i] <- sum((y-yest)^2)

			if(debug)
			{
				o <- order(x)
				lines(x[o], yest[o], col="blue")
			}
		}
		# determine the gamma that minimizes SS
		min.index <- which(SS==min(SS))[1]
		start$gamma <- gammas[min.index]
	}

	if(debug)
	{
		if(include.gamma)
			yy <- Emax.fun(xcand, E0=start$E0, Emax=start$Emax, 
				EC50=start$EC50, gamma=start$gamma)
		else
			yy <- Emax.fun(xcand, E0=start$E0, Emax=start$Emax, 
				EC50=start$EC50)

		lines(xcand, yy , col="red", lwd=2)

		cat("\nDebugging mode in starting values.\n\n")
		cat("\nStarting values:\n")
		print(unlist(start))
		browser()
	}

	# ---
	# return the result
	
	return(start)
}

