
# ---
# Graph of model fit, CI and data

Emax.xyplot <- function(
	yvar, xvar, # variable names
	x, y, # data (x and y)
	df.plot, 	# plotting data frame for fit and CI

	grps, # grouping variable
	xaxis.at, yaxis.at, # axis tickmarks
	xaxis.lab, # labels for tickmarks

	col.indicator,	# color for special tickmarks
	col.indicator.text,

	lines.col, lines.lwd,
	points.col, points.pch="X",

	main="", 
	x.indicate, y.indicate,  
	x.ind2, y.ind2,  
	str.ind2,

	logscale.plot,
	logscale.fit,

	align.str.xy,

	xlim, ylim, 

	ylab
)

{
	# Define a customized (double) axis
	# See Sarkar, Lattice, p. 149

	axis.mine <- function(side, ...)
	{
		if((length(y.ind2) > 0 )& (side=="right"))
		{
			# Remove y-values outside the current panel's limits
			# Rare situations where Emax/EC50 are very large and far outside
			# the data range might yield huge values to indicate, causing the
			# axis function to crash (e.g., a linear relationship with an 
			# Emax fit)
			lims <- current.panel.limits()
			ok <- (y.ind2 >= lims$ylim[1]) & (y.ind2 <= lims$ylim[2])
			if(sum(ok) > 0)
				panel.axis(side=side, outside=TRUE, 
					at=y.ind2[ok], labels=str.ind2[ok],
					line.col=col.indicator, text.col=col.indicator.text)
			else
				axis.default(side=side, ...)
		}
		else
			axis.default(side=side, ...)
	}

	graph <- xyplot(
		y ~ x.plot, # Note that we plot the fit and add the data. 

		groups=what, # (fit, ci lower, ci upper)
		type="l",
		data=df.plot,

		obs.x=x,
		obs.y=y,

		col=lines.col, 
		lwd=lines.lwd, 

		col.points=points.col,
		pch.points=points.pch,
		grps=grps,

		xlab="", # xlab is used in second graph, and that suffices.
		ylab=ylab,
		main=main, 
		# sub=sub, do not add further lines, they stretch into bwplot

		axis=axis.mine, # customized axis

		scales=list(
			# sync settings for this x-axis with settings for bwplot x-axis
			x=list(at=xaxis.at, labels=xaxis.lab, limits=xlim),
			y=list(
				# include a long dummy label to align y-axes of both graphs(!)
				at=yaxis.at, 
				lab=format(c(align.str.xy, yaxis.at), justify="right")[-1],
				limits=ylim
			) 
		),

		# additional (x, y) values to indicate in graph
		x.ind=x.indicate,	# without labels at y-axis
		y.ind=y.indicate,

		x.ind.label=x.ind2,	# with labels at y-axis (e.g., "80%")
		y.ind.label=y.ind2,
		label.ind=str.ind2,

		# layout of two graphs in one
		plot.args=list(position=c(0, 0.25, 1, 1), more=TRUE),

		panel=function(x, y, ..., obs.x, obs.y, grps, 
			col.points, pch.points,
			x.ind, y.ind, x.ind.label, y.ind.label, label.ind
		)
		{
			# auxiliary lines
			# panel.grid(h=-1, v=-1, col="lightgray")
			# panel.grid still sticks to default axis ticks, useless.

			panel.abline(h=yaxis.at, v=xaxis.at, col="lightgray")

			lims <- current.panel.limits()

			# add additional values to show (user-specified y-values)
			if(length(y.ind) > 0)
				for(i in 1:length(y.ind))
					panel.lines(c(lims$xlim[1], x.ind[i], x.ind[i]), 
						c(y.ind[i], y.ind[i], lims$ylim[1]),
						col=col.indicator
					)

			# pre-specified Emax values, e.g., EC20, EC80
			xlimits <- current.panel.limits()$xlim
			xpos <- xlimits[1] - 0.01*(xlimits[2]-xlimits[1])

			for(i in 1:length(y.ind.label))
			{
				# lines from y-pos to x-pos "around the corner"
				panel.lines(c(lims$xlim[2], x.ind.label[i], x.ind.label[i]), 
					c(y.ind.label[i], y.ind.label[i], lims$ylim[1]),
					col=col.indicator, lwd=1
				)
			}

			# ---
			# observed data last to overlay everything else

			ymax <- current.panel.limits()$ylim[2]
			ymin <- current.panel.limits()$ylim[1]
			xmax <- current.panel.limits()$xlim[2]
			xmin <- current.panel.limits()$xlim[1]

			obs.y.cut <- ifelse(obs.y     > 0.99*ymax, 0.99*ymax, obs.y)
			obs.y.cut <- ifelse(obs.y.cut < 1.01*ymin, 1.01*ymin, obs.y.cut)

			obs.x.cut <- ifelse(obs.x     > xmax, xmax, obs.x)
			obs.x.cut <- ifelse(obs.x.cut < xmin, xmin, obs.x.cut)

			# color-code dose groups
			grp <- as.numeric(factor(grps)) # get numbers 1, 2, 3, ...

			panel.xyplot(obs.x.cut, obs.y.cut, 
				col=col.points[grp], pch=pch.points[grp])

			# fit and CI
			panel.xyplot(x, y, ...)

		} # end panel function
	) # end xyplot

	return(graph)

}

