
# Derive baseline value by subject and add it as a new variable to the
# data set.
# Run the function without arguments to see an example.

baseline.get <- function(

	data=data.frame(
		ID=rep(LETTERS[1:5], rep(3, 5)),
		time=rep(0:2, 5),
		response=1:15
	),

	# names of variables in the data set
	varname="response", # name of variable to get baseline values for
	ID="ID",
	time="time",

	time.bl=0, 
		# time of baseline measurement (can also be -1 or anything)

	varname.BL=paste(varname, ".BL", sep=""), 
		# name of new BL variable that is inserted into the data set

	BL.pos=which(names(data)==varname)+1,
		# index of column for the BL variable (to be inserted)
		# default: insert BL variable right after source variable

	quiet=FALSE	
		# if TRUE, no messages will be shown.
)
{
	# ---
	# Initialization

	if(quiet)
		msg <- function(...) { } # does nothing
	else
		msg <- function(...) { cat(..., "\n", sep="") } # print to screen

	msg("Baseline derivation for ", varname, " in ", deparse(substitute(data)))

	if(any(names(data)==varname.BL))
		stop("Variable name ", varname.BL, " exists already in data set.")

	# ---
	# Determine baseline records, keep only those, and merge the baseline
	# data set back into the data set.

	is.BL <- data[, time]==time.bl
	data.bl <- data[is.BL, c(ID, varname)]
	names(data.bl) <- c(ID, varname.BL)

	# Check if any ID has more than one baseline record (should not be)
	dupl.ID <- duplicated(data.bl[, ID])
	if(any(dupl.ID))
		warning("Duplicate baseline values for ID ", 
			paste(data.bl[dupl.ID, ID], collapse=", "))

	data.new <- merge(data, data.bl, all=TRUE) # merge by ID	

	# ---
	# Move BL variable to the desired position (column)
	# Take it out of the data set and insert it into the desired position

	BL.data <- data.new[, varname.BL, drop=F]	# store data in new variable
	which <- which(names(data.new)==varname.BL)
	data.new <- data.new[, -which]	# take out from the data set 

	if(BL.pos==1)
		data.new <- cbind(BL.data, data.new)

	if(BL.pos > 1 & BL.pos <= ncol(data.new))
		data.new <- cbind(
			data.new[, 1:(BL.pos-1), drop=F], 
			BL.data, 
			data.new[, BL.pos:ncol(data.new), drop=F]
		)

	if(BL.pos > ncol(data.new))
		data.new <- cbind(data.new, BL.data)	# just append

	# ---
	# Final error checks

	if(nrow(data) != nrow(data.new))
		stop("Baseline derivation seems to have gone wrong, ",
			"additional rows detected.")

	if(ncol(data) != ncol(data.new)-1)
		stop("Baseline derivation seems to have gone wrong, ",
			"expected addition of one column.")

	# ---
	# The End.

	o <- order(data.new[, ID], data.new[, time]) # restore order after merging
	data.new <- data.new[o, ]

	return(data.new)
	
}

