# ---
# Customized function to plot observed data and fit for a GLM
# Pass an object of class "glm" to this function.
# Restrictions: You must use data= in the call to the glm (such that the 
# result returned contains the data in the $data list component).


glm.plot <- function(fit, ..., main="GLM data and fit", 
	xlab=xvar, ylab=yvar, # use lazy evaluation for axis labels (see code)
	ylim=c(-0.025, 1.025),
	groups, # if you want color codes for different groups such as dose groups
	alpha=0.95	# significance level for two-sided CI
)
{
	require(lattice)

	# ---
	# Error checks

	if(alpha < 0.5)
	{
		warning("alpha=", alpha, " changed to alpha=", 1-alpha)
		alpha <- 1-alpha
	}

	pred <- predict(fit, type="response")
	obs  <- fit$data

	tmp <- attr(fit$terms, "dataClasses")
	yvar <- names(tmp)[1]
	xvar <- names(tmp)[2]
	if(length(tmp) > 2)
		stop("Not written to handle more than one x-variable.")

	xdata <- fit$data[[xvar]]
	ydata <- fit$data[[yvar]]

	# Graph with confidence intervals (!)
	# See DoseFinding:::plot.DRMod

	newx <- pretty(xdata, n=50)
	df <- data.frame(newx)
	names(df) <- xvar

	# predicted response
	pred.grid <- predict(fit, newdata=df, type="response", se.fit=TRUE)

	# predicted response on the linear scale
	# (transformation of predicted value and boundaries follows below)

	pred.grid <- predict(fit, newdata=df, se.fit=TRUE)

	df$y <- pred.grid$fit
	names(df)[1] <- "x"

	# CI
	tval <- qt(1-(1-alpha)/2, df=summary(fit)$df.residual)
	df$low <- df$y - pred.grid$se.fit*tval
	df$high<- df$y + pred.grid$se.fit*tval

	# Logistic transformation of boundaries
	df$y    <- plogis(df$y)
	df$low  <- plogis(df$low)
	df$high <- plogis(df$high)

	df2 <- data.frame(
		x=df$x, 
		y=c(df$y, df$low, df$high),
		what=rep(c("mean", "low", "high"), rep(nrow(df), 3))
	)

	gr <- xyplot(y ~ x, data=df2, groups=what, 
		obs.x=xdata, obs.y=ydata, 
		type="l", col="blue",
		xlab=xlab, ylab=ylab, main=main,
		ylim=ylim,
		..., 
		panel=function(x, y, ..., obs.x, obs.y)
		{
			panel.grid(h=-1, v=-1, col="lightgray")
			panel.points(obs.x, obs.y, pch="|", col="black")
			panel.xyplot(x, y, ...)
		}
	)

	# Statistics with %CV inserted (!)
	coef <- coef(summary(fit))
	coef <- cbind(coef, 
		"%CV"=coef[, "Std. Error"] / abs(coef[, "Estimate"]) * 100
	)
	print(coef)

	return(gr)
}

