# Graph a profile of means and associated standard deviations or quantiles.
# Allows for stratification, ie one panel per dose group or study or whatever
# the grouping variable might be.
# Andreas Krause, March 2009

# Use the Theophylline data as default.
# This way you can call the function without arguments to see an example.
# Just enter:
# modeling.meansplot()

# data(Theoph)	# Now part of R at time of startup.

modeling.meansplot <- function(

	# specify the data here. Default: Theophylline.
	x=round(Theoph$Time/2)*2, 
	y=Theoph$conc,
	groups1=cut(Theoph$Dose, c(3, 5, 7)),
	groups2=cut(Theoph$Wt, c(50, 70, 100), labels=c("BW < 70", "BW >= 70")),

	# Function to calculate the means. 
	# Alternatively use median or anything else
	agg.fun=mean,

	# if use.quanti
	bounds="sdv",	
		# "sdv" shows standard deviations, 
		# "quantiles" shows quantiles of the data as polygons around the means

	quant=0.1,	# quantiles to show for upper/lower boundaries
	sigmas=2,	# this number of sigmas around the mean for boundaries

	# labels for axes
	xlab=deparse(substitute(x)),
	ylab=deparse(substitute(y)),

	# default: sort groups from top left corner
	as.table=TRUE,

	# indicator flag if Theophylline data is used or not
	# (string "Theoph" occurs in x and y)
	use.Theo=length(
		grep("Theoph", 
		c(deparse(substitute(x)), deparse(substitute(y)))
	))==2,

	# define the layout
	group.vars=ifelse(use.Theo, 2, sum(!missing(groups1), !missing(groups2))),
	layout=switch(as.character(group.vars),
		"0"=c(1, 1),
		"1"=c(length(unique(groups1)), 1),
		"2"=c(length(unique(groups1)), length(unique(groups2))),
		99 #stop("layout calculation went wrong.")
	),

	# vertical lines desired? 
	# Set vlines=seq(0, max(y), by=24) to get 24 (hour) intervals as 
	# vertical lines.

	hlines,
	vlines,

	# anything else you want to pass to xyplot
	# Examples: 
	# scales=list(relation="free") # adapts ranges to group's ranges
	# layout=c(5, 1) to have 5 columns and just one row
	# ylim=c(-3, 10) to set specific y-axis limits
	...
)
{
	# ---
	# Init

	# quant must be a scalar between 0 and 0.5 (lower bound)
	if(quant < 0 | quant > 1)
		stop("quant must be in [0, 1]")

	quant <- min(c(quant, 1-quant), na.rm=TRUE)

	if(!is.element(bounds, c("sdv", "quantiles")))
		stop("bounds must be either \"sdv\" or \"quantiles\"")

	if(sigmas <= 0)
		stop("sigmas must be larger than 0")

	z <- list(x=x, y=y)
	if(!missing(groups1)) z$groups1 <- groups1
	if(!missing(groups2)) z$groups2 <- groups2

	for(i in 1:length(z))
		if(length(z[[i]]) ==0) stop(names(z)[i], " is of length 0.")

	# See that all arguments have the same number of records.
	# groups might be a data frame with more than one variable

	z.n <- sapply(z, function(x) nrow(as.data.frame(x)))
	if(length(unique(z.n)) > 1)
		stop("Arguments x, y, groups have different lengths: ", 
			paste(z.n, collapse=", "))


	if(missing(hlines)) hlines <- NULL
	if(missing(vlines)) vlines <- NULL

	# ---
	# Do the calculations
 
	# X is the matrix for calculating means/medians by X variables
	X <- data.frame(x=x)
	if(!missing(groups1) | use.Theo) X <- data.frame(X, groups1)
	if(!missing(groups2) | use.Theo) X <- data.frame(X, groups2)

	m <- aggregate(list(Y=y), X, agg.fun, na.rm=TRUE)

	sdv.calc <- function(x)
	{
		x <- x[!is.na(x) & is.finite(x)]
		n <- length(x)
		return(sqrt(var(x)/n))
	}

	sdv <- aggregate(list(Y=y), X, sdv.calc)

	quants.low <- aggregate(list(Y=y), X, 
		function(x, q) quantile(x, q, na.rm=T), q=quant)

	quants.high <- aggregate(list(Y=y), X, 
		function(x, q) quantile(x, q, na.rm=T), q=1-quant)


	# Put together a single data set with all aggregated information
	data <- data.frame(
		m[, -ncol(m), drop=F],	# without mean column
		m=m$Y,
		sdv=sdv$Y,
		q.low=quants.low$Y,
		q.high=quants.high$Y,
		sdv.low =m$Y - sigmas * sdv$Y,
		sdv.high=m$Y + sigmas * sdv$Y
	)

	# ---
	# Do the graph

	# put together the formula

	formula <- "m ~ x"

	if(!missing(groups1) | use.Theo)
		formula <- paste(formula, "| format(groups1)")
	if(!missing(groups2) | use.Theo)
		formula <- paste(formula, "+ format(groups2)")

	formula <- as.formula(formula)

	# And here comes the graph

	graph <- xyplot(formula, 
		data=data, 
		alldata=data,
		what.bounds=bounds,	# pass argument from this function to panel

		xlab=xlab,
		ylab=ylab,

		as.table=as.table,
		layout=layout,

		hlines=hlines,	# vertical lines
		vlines=vlines,	# vertical lines

		...,	# anything else that was passed to this function
		panel=function(x, y, ..., subscripts, alldata, what.bounds,
			hlines, vlines
		)
		{
			args <- list(...)

			# the current data set with all other variables, too
			D <- alldata[subscripts, ]

			# horizontal and vertical lines
			if(length(hlines) > 0)
				panel.abline(h=hlines, col="lightgray")

			if(length(vlines) > 0)
				panel.abline(v=vlines, col="lightgray")

			# put together a single polygon, concatenating upper and lower
			# boundaries

			x2 <- c(D$x, rev(D$x))

			if(what.bounds=="sdv")
				y2 <- c(D$sdv.low, rev(D$sdv.high))

			if(what.bounds=="quantiles")
				y2 <- c(D$q.low, rev(D$q.high))

			# panel.polygon does weird things if there are missing data
			# Now we "jump over" this point (call it interpolation) but 
			# actually there should be two polygons, 
			# left and right of the missing, in the graph.

			ok <- !is.na(x2) & !is.na(y2)
			x2 <- x2[ok]
			y2 <- y2[ok]

			# draw the polygon (shaded area) for the boundaries
			panel.polygon(x2, y2, density=1, col="lightyellow")

			# polygon of means
			panel.points(D$x, D$m, type="b", col="red", pch=16, ...)
		}
	)

	# ---
	# The End.

	return(graph)
}

