# PK profiles stratified in different ways.
# The function uses the theophylline data that comes with R.
# Doses are all the same (320 mg) except one subject (9) and
# the dose reported here is mg/kg.
# However, it serves our purpose of illustration.

pkprofiles <- function(
	data=Theoph,

	xlab="time after dose (h)",
	ylab="drug concentration (ng/mL)"
)
{
	# Convert Subject from ordered factor to numeric
	data$Subject <- as.numeric(as.character(data$Subject))

	# Create dose groups low and high
	# Note: the leading space makes " low" come before "high" in alphabetical
	# order (factor levels)

	dose.group <- ifelse(data$Dose < 4.5, " low dose", "high dose")

	# Create weight groups
	# Note: the leading space makes " low" come before "high" in alphabetical
	# order (factor levels)

	weight.group <- ifelse(data$Wt< 70, " low body weight", "high body weight")

	# Make up fed/fasted data 

	data$ID <- ifelse(data$Subject <= 6, data$Subject, data$Subject -6)
	data$food.status <- ifelse(data$Subject <= 6, "fed", "fasted")

	# Insert a line with missing values in conc such that lines get
	# interrupted when ID is used as grouping variable
	# (it will otherwise interconnect fed and fasted)

	tmp <- data[data$food.status=="fasted", ]
	tmp <- tmp [!duplicated(tmp$ID), ]
	tmp$conc <- rep(NA, nrow(tmp))
	tmp$Time <- rep(max(data$Time, na.rm=TRUE)+0.001, nrow(tmp))
	data <- rbind(data, tmp)
	data <- data[order(data$ID, data$food.status, data$Time), ]

	# ---
	# Graphing

	require(lattice)

	graphs <- list()

	graphs$all.in.one <- xyplot(conc ~ Time, groups=ID, 
		xlab=xlab, ylab=ylab, 
		data=data, type="b")

	graphs$individual <- xyplot(conc ~ Time | format(ID), 
		groups=food.status,
		xlab=xlab, ylab=ylab, 
		as.table=TRUE, data=data, type="b")

	graphs$by.food <- xyplot(conc ~ Time | food.status, groups=Subject,
		xlab=xlab, ylab=ylab, 
		data=data, type="b")

	graphs$by.weight <- xyplot(conc ~ Time | weight.group, groups=Subject,
		xlab=xlab, ylab=ylab, 
		data=data, type="b")

	# Combination of two graphs

	graphs[["combo"]] <- list(
		gr1=update(
			graphs$individual,
			plot.args=list(position=c(0, 0, 0.5, 1.0), more=TRUE)
		),
		gr2=update(
			graphs$by.food,
			plot.args=list(position=c(0.5, 0, 1.0, 1.0), more=FALSE)
		)
	)

	# ---
	# The End.

	return(graphs)
}

