# Show safety data as survival plots.
# Data: subject ID and first occurence of an AE.

x <- read.table("safety.dat", header=T, sep="\t")

safety.as.survival <- function(
	data=x,
	maxtime=200,	# evaluation from time 0 to time maxtime

	ID="ID",
	time="time",	# name of time variable in data set
	trt="treatment",
	AE="AE",
	censored="censored",
	event="event"	# 1=AE, anything else is ignored (use if censored=1 w/o AE)
)
{
	# ---
	# Error checks

	for(what in c("time", "trt", "AE", "censored"))
		if(!is.element(get(what), names(data)))
			stop(what, "=", get(what), " seems wrong. ",
				"No such variable in data set.")

	# ---
	# create evaluation frame for each event day

	unique <- lapply(data[c(time, AE, trt)], 
		function(x) sort(unique(x))
	)
	unique[[time]] <- sort(unique(c(0, unique[[time]], maxtime)))

	# triple each time point to (t[i]-epsilon, t[i], t[i]+epsilon) such that
	# we actually get stair steps instead of a polygon with diagonal lines

	unique[[time]] <- sort(
		c(unique[[time]]-0.01, unique[[time]], unique[[time]]+0.01))

	n <- sapply(unique, length)	# number of unique values

	# all combinations of time, AE, and treatment

	df <- data.frame(
		time=rep(unique[[time]], n[AE]*n[trt]),
		surv=rep(-1, n[time] * n[AE] * n[trt]),
		AE  =rep(rep(unique[[AE]], rep(n[time], n[AE])), n[trt]),
		trt =rep(unique[[trt]], rep(n[time]*n[AE], n[trt]))
	)	

	names(df) <- c(time, "surv", AE, trt)

	# ---
	# Count the survival status for each AE and trt at each time point

	for(i in 1:nrow(df))
	{
		which <- 
			data[[trt]] == df[[trt]][i] & 
			data[[AE]]==df[[AE]][i] &
			data[[time]] <= df[[time]][i] &
			data[[censored]]==0

		subset <- data[which, ]
		events <- sum(subset[[event]]==1)

		nsubjects <- length(unique(data[[ID]][data[[trt]]==df[[trt]][i]]))

		at.risk <- nsubjects - sum(subset[[censored]]==1)
		df$surv[i] <- 1 - events/ at.risk

	}

	# ---
	# Graph

	require(lattice)

	graph <- xyplot(
		as.formula(paste("surv ~", time, "|", AE)),
		data=df,
		groups=get(trt),
		type="l",
		ylim=c(-0.05, 1.05),
		layout=c(n[AE], 1),

		xlab="time",
		ylab="percentage of subjects without AE",

		panel=function(x, y, ...)
		{
			panel.xyplot(x, y, ...)

			args <- list(...)
			grp <- args$groups[args$subscripts]
			# calculate the y-value where x is max for each group
			z <- split(data.frame(x, y), grp)
			last <- lapply(z, function(Z) Z[Z$x==max(Z$x), ][1, ])

			# add text to graph
			labs <- paste("\n", names(last), sep="")	# add a line feed
		
			colors <- trellis.par.get("superpose.line")$col
			for(i in 1:length(last))
			panel.text(last[[i]]$x, last[[i]]$y, labs[i], 
				cex=0.7, adj=1, col=colors[i])
		}
	)

	# ---
	# The End

	return(list(data=df, graph=graph))

}

