# Bar charts to compare two (or more) treatment arms.
# Particular emphasis on the ordering of the categories.

safety.barchart <- function(
	datafile="safety.barchart.dat",
	order.by=c("active", "placebo")	# variables to sort by
)
{
	x <- read.table(file=datafile, header=TRUE, sep="\t")

	# format into Trellis structure

	trt <- names(x)
	trt <- trt[(length(trt)-1):length(trt)]	# last two col names

	df <- data.frame(
		category = rep(x$category, length(trt)),	
		event = rep(x$event, length(trt)),	
			# the safety event (headache, etc.)
		result= unlist(x[, c("active", "placebo")]),	# the numbers
		compound = rep(trt, rep(nrow(x), length(trt)))
	)

	# ---

	require(lattice)

	graphs <- list()

	# sort by event within each treatment group

	for(what in c("as.is", levels(df$compound), "abs.diff", "rel.diff"))
	{
		compounds <- levels(df$compound)

		if(what=="abs.diff")
		{
			trt1 <- df[df$compound==compounds[1], ]
			trt2 <- df[df$compound==compounds[2], ]
			diff <- trt1$result - trt2$result
			o <- order(diff)
			which <- df$compound==compounds[1]	# arbitrary choice
		} 
		else if(what=="rel.diff")
		{
			trt1 <- df[df$compound==compounds[1], ]
			trt2 <- df[df$compound==compounds[2], ]
			ratio <- trt1$result / trt2$result
			o <- order(ratio)
			which <- df$compound==compounds[1]	# arbitrary choice
		}
		else if(what=="as.is")
		{
			which <- df$compound==compounds[1]	# arbitrary choice
			o <- 1:length(levels(df$event))	# order 1...n
		}
		else
		{
			which <- df$compound==what
			o <- order(df$result[which])
		}

		ev.levels <- as.character(df$event[which])[o]
		df$event <- factor(as.character(df$event), levels=ev.levels)

		# do the graph

		graphs[[what]] <- dotplot(
			event ~ result,
			data=df,
			groups=compound,
			scales=list(
				y=list(cex=1.1)	# larger labels
			),
			col=c("blue", "red"),
			pch=c("A", "P"), 
			# pch=16,
			cex=1.5,

			xlab=paste("frequency (percent)", "\n", 
				switch(what,
					"as.is"="",
					"abs.diff"="sorted by absolute difference",
					"rel.diff"="sorted by relative difference",
					"active"="sorted by active",
					"placebo"="sorted by placebo",
					stop("Cannot compose xlab.")
				),
				sep=""
			)
		)
	}

	# by category of AE

	graphs[["by category"]] <- dotplot(
		event ~ result | category,
		data=df,
		groups=compound,
		scales=list(
			y=list(cex=1.1)	# larger labels
		),
		col=c("blue", "red"),
		pch=c("A", "P"), 
		# pch=16,
		cex=1.5,
		xlab=paste("frequency (percent)", "\n", 
			switch(what,
				"as.is"="",
				"abs.diff"="sorted by absolute difference",
				"rel.diff"="sorted by relative difference",
				"active"="sorted by active",
				"placebo"="sorted by placebo",
				stop("Cannot compose xlab.")
			),
			sep=""
		)
	)

	# Combination of multiple graphs into one object

	graphs[["combo"]] <- list(
		gr1=update(graphs$active, 
			plot.args=list(position=c(0.0, 0.0, 0.33, 1.0), more=TRUE)),
		gr2=update(graphs$rel.diff, 
			plot.args=list(position=c(0.3, 0.0, 0.63, 1.0), more=TRUE)),
		gr3=update(graphs[["by category"]],
			plot.args=list(position=c(0.6, 0.0, 1.0, 1.0), more=FALSE))
	)

	# ---
	# The End.

	return(graphs)
}

